'use strict';

Service.prototype.setService = function (from) {

	this.from = from || 'other';
	this.domain = from;
	this.url = '';
	this.mobileURL = '';
	this.encoding = null;
	this.extraRequired = false;
	this.passRequired = false;
	this.cookiesRequired = false;
	this.noCache = false;
	this.postType = null;
	this.mobileUserAgent = false;
	this.desktopUserAgent = false;
	this.oneByOne = false;
	this.ignoreText = false;
	this.countToShip = false;
	this.useItemCache = false;
	this.tokenID = null;
	this.minOSFeatureLevel = 0;

	switch (from) {

		case 'www.amazon.com':
		case 'www.amazon.ca':
		case 'www.amazon.co.uk':
		case 'www.amazon.co.jp':
		case 'www.amazon.de':
		case 'www.amazon.at':
		case 'www.amazon.es':
		case 'www.amazon.fr':
		case 'www.amazon.in':
		case 'www.amazon.it':
			if (this.domain == 'www.amazon.at') this.domain = 'www.amazon.de';
			this.extraRequired = true;
			this.desktopUserAgent = true;
			this.cookiesRequired = true;
			this.countToShip = true;
			this.oneByOne = true;
			this.useItemCache = true;
			break;

		case 'www.amazon.com/url':
			var result = this.extra.match(/^https:\/\/((?:www|smile)\.amazon\.(?:co\.)?[a-z]+)\//i);
			this.domain = (result) ? result[1] : 'www.amazon.com';
			this.extraRequired = true;
			this.desktopUserAgent = true;
			this.cookiesRequired = true;
			this.countToShip = true;
			this.oneByOne = true;
			break;

		case 'www.apple.com':
		case 'www.apple.com/zip':
		case 'www.apple.com/japan':
		case 'www.apple.com/japanzip':
		case 'www.apple.com/hk':
			switch (this.from) {
				case 'www.apple.com/zip': this.from = 'www.apple.com'; break;
				case 'www.apple.com/japanzip': this.from = 'www.apple.com/japan'; break;
				default: break;
			}
			this.desktopUserAgent = true;
			this.extraRequired = true;
			this.countToShip = true;
			this.minOSFeatureLevel = 5;
			break;

		case 'www.chronopost.fr':
			this.desktopUserAgent = true;
			this.minOSFeatureLevel = 5;
			break;

		case 'www.dhl.com':
		case 'track.dhl-usa.com':
		case 'www.dhl.co.uk':
			this.from = 'www.dhl.com';
			break;

		case 'nolp.dhl.de':
		case 'nolp.dhl.de/int':
			this.from = 'nolp.dhl.de';
			break;

		case 'www.fedex.com':
		case 'spportal.fedex.com':
			this.from = 'www.fedex.com';
			this.minOSFeatureLevel = 5;
			break;

		case 'www.laposte.fr':
			this.tokenID = 'api.laposte.fr';
			break;

		case 'www.ontrac.com':
			this.postType = 'application/json';
			break;

		case 'www.poste.it':
		case 'www.poste.it/posta1':
		case 'www.poste.it/pacco1':
		case 'www.poste.it/maxi':
		case 'www.poste.it/pacco':
		case 'www.poste.it/ems':
			this.postType = 'application/json';
			this.oneByOne = true;
			this.minOSFeatureLevel = 5;
			break;

		case 'www.postnl.nl':
		case 'www.tntpost.nl':
			this.extraRequired = true;
			break;

		case 'www.post.ch':
			this.encoding = 'iso-8859-1';
			this.minOSFeatureLevel = 5;
			break;

		case 'www.tntexpress.com.au':
			this.desktopUserAgent = true;
			this.minOSFeatureLevel = 5;
			break;

		case 'www.ups.com':
		case 'wwwapps.ups.com':
			this.from = 'www.ups.com';
			this.postType = 'application/json';
			this.minOSFeatureLevel = 5;
			break;

		case 'www.usps.com':
			this.oneByOne = true;
			this.minOSFeatureLevel = 5;
			break;

		case 'www.aramex.com':
		case 'em.canadapost.ca':
		case 'www.lasership.com':
		case 'www.parcelforce.com':
			this.minOSFeatureLevel = 5;
			break;

		case 'other':
			this.url = this.extra;
			break;

	}

}

Delivery.prototype.startUpdate = function (service, childNo) {

	var isChild = (childNo !== false);

	switch (service.from) {

		case 'www.adobe.com':

			var no = service.no.replace(/[^A-Za-z0-9]/g,'');
			service.url = 'https://accounts.adobe.com/orders/'+no;	
			this.notSupported(isChild);
			this.updateItems();
			break;

		case 'www.amazon.com':
		case 'www.amazon.ca':
		case 'www.amazon.co.uk':
		case 'www.amazon.co.jp':
		case 'www.amazon.de':
		case 'www.amazon.at':
		case 'www.amazon.es':
		case 'www.amazon.fr':
		case 'www.amazon.in':
		case 'www.amazon.it':

			var no = service.no.replace(/[^0-9]/g,'').replace(/([0-9]{3})([0-9]{7})([0-9]{7})/,'$1-$2-$3');
			if (!no.match(/[0-9]{3}-[0-9]{7}-[0-9]{7}/)) {
				service.count = 0;
				var errorText = 'You must enter an Amazon order number, not a tracking number.';
				this.updateFailed(DeliveryError.notFound, errorText);
				this.updateItems();
			} else {
				service.url = 'https://'+service.domain+'/gp/css/order-details?orderId='+encodeURIComponent(no);
				if (this.itemCache && this.itemCache.length > 0) {
					logMessage(1,'Using item cache '+no);
					this.loadItemCache(this.getTracker, 'amazon-track');
				} else if (manualSignIn[service.from+'-'+service.extra]) {
					logMessage(1,'startUpdate skipping '+service.from);
					service.count = 0;
					this.updateFailed(DeliveryError.signIn, null);
					this.updateItems();
				} else {
					logMessage(1,'Item cache unavailable '+no);
					this.loadPage(service.url, this.getAmazon);
				}
			}
			break;

		case 'www.amazon.com/url':

			var urlRegEx = /^https:\/\/((?:www|smile)\.amazon\.(?:co\.)?[a-z]+)\/(progress-tracker\/package|gp\/your-account\/ship-track|gp\/css\/shiptrack|gp\/your-account\/order-history\/item)[\/?%]/i;
			var url = false;
			if (service.extra) {
				if (service.extra.match(urlRegEx)) {
					url = service.extra;
				} else {
					var data = getData(service.extra);
					var dataURL = data['U'];
					if (dataURL && dataURL.match(urlRegEx)) url = dataURL;
				}
				if (url && !url.match(/[&?]packageIndex=/i)) {
					url += '&packageIndex=';
				}
			}
			if (url) {
				service.url = url;
				var options = new RequestOptions('amazon-track', childNo);
				this.loadPage(service.url, this.getTracker, options);
			} else {
				service.count = 0;
				var errorText = 'Enter a link to a “Track Package” page on Amazon.';
				this.updateFailed(DeliveryError.notFound, errorText);
				this.updateItems();
			}
			break;

		case 'www.apple.com':
		case 'www.apple.com/zip':

			service.url = 'https://secure.store.apple.com/us/order/guest/'+encodeURIComponent(service.no)+'/'+doublePlusEncode(service.extra);
			this.loadPage(service.url, this.getAppleJSON);
			break;

		case 'www.apple.com/japan':
		case 'www.apple.com/japanzip':
		case 'www.apple.com/hk':

			var countryCode = (service.from == 'www.apple.com/hk') ? 'hk-zh' : 'jp';
			service.url = 'https://secure.store.apple.com/'+countryCode+'/order/guest/'+encodeURIComponent(service.no)+'/'+doublePlusEncode(service.extra);
			this.loadPage(service.url, this.getApple);
			break;

		case 'www.aramex.com':

			service.url = 'https://www.shopandship.com/en/tracking/'+encodeURIComponent(service.no);
			var url = 'https://www.shopandship.com/api/packages/'+encodeURIComponent(service.no)+'/singlePackage?awbs='+encodeURIComponent(service.no)+'&count=1&language=en&useXMLTracking=false';
			var options = new RequestOptions('aramex', childNo);
			this.loadPage(url, this.getJSON, options);
			break;

		case 'auspost.com.au':

			service.url = 'https://auspost.com.au/mypost/track/#/details/'+encodeURIComponent(service.no);
			this.notSupported(isChild);
			this.updateItems();
			break;
	
		case 'em.canadapost.ca':
	
			var options = new RequestOptions('canada-post', childNo);
			options.lang = preferredLanguage('en','fr');
			service.url = 'https://www.canadapost.ca/trackweb/'+options.lang+'#/search?searchFor='+
				encodeURIComponent(service.no);
			var url = 'https://www.canadapost.ca/trackweb/rs/track/json/package/'+
				encodeURIComponent(service.no)+'/detail';
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.chronopost.fr':
	
			var options = new RequestOptions('chronopost', childNo);
			var no = service.no.replace(/ /g,'');
			options.lang = remapString(preferredLanguage('en','fr'), {'en':'gb'});
			service.url = 'https://www.chronopost.fr/fr/chrono_suivi_search?listeNumerosLT='+encodeURIComponent(no)+'&lang='+options.lang;
			options.referer = service.url;
			var url = 'https://www.chronopost.fr/tracking-no-drupal/suivi-colis?listeNumerosLT='+encodeURIComponent(no)+'&langue='+options.lang;
			this.loadPage(url, this.getHTMLJSON, options);
			break;
	
		case 'www.dhl.com':
		case 'track.dhl-usa.com':
		case 'www.dhl.co.uk':
	
			var options = new RequestOptions('dhl', childNo);
			var no = encodeURIComponent(service.no.replace(/ /g,''));
			service.url = 'https://www.dhl.com/content/g0/en/express/tracking.shtml?brand=DHL&AWB='+no;
			var url = 'https://www.dhl.com/shipmentTracking?AWB='+no+'&countryCode=g0&languageCode=en';
			options.service = service;
			this.loadPage(url, this.getJSON, options);
			break;
	
		case 'www.dhlglobalmail.com':

			var lang = preferredLanguage();
			service.url = 'https://webtrack.dhlglobalmail.com/?mobile=false&trackingnumber='+
				encodeURIComponent(service.no)+'&locale='+lang;
			this.notSupported(isChild);
			this.updateItems();
			break;
	
		case 'nolp.dhl.de':
		case 'nolp.dhl.de/int':

			var options = new RequestOptions('dhl-germany', childNo);
			var no = encodeURIComponent(service.no.replace(/[ .]*/g,''));
			options.lang = preferredLanguage('en','de');
			service.url = 'https://www.dhl.de/'+options.lang+'/privatkunden/pakete-empfangen/verfolgen.html?piececode='+encodeURIComponent(no);
			var url = 'https://www.dhl.de/int-verfolgen/search?lang='+options.lang+'&domain=de&piececode='+encodeURIComponent(no);
			options.service = service;
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.dpd.com':

			var options = new RequestOptions('dpd', childNo);
			var no = service.no.replace(/[^A-Z0-9]/gi,'');
			var shortNo = no;
			if (shortNo.length == 15) shortNo = shortNo.substr(0,14); // remove check digit
			var locale = preferredLocale('en_US','de_DE');
			service.url = 'https://tracking.dpd.de/status/'+locale+'/parcel/'+no;
			var url = 'https://tracking.dpd.de/rest/plc/'+locale+'/'+shortNo;
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.dpd.co.uk':
	
			service.url = 'https://www.dpd.co.uk/deliverysearch/?parcel='+encodeURIComponent(service.no);
			this.notSupported(isChild);
			this.updateItems();
			break;

		case 'www.fedex.com':

			var no = service.no.replace(/^([0-9]+)E$/,'$1');
			var locale = remapString(
				preferredLocale('en_US','de_DE','fr_FR','ja_JP','it_IT','sv_SE','da_DK','nl_NL','no_NO','nb_NO'),
				{'nb_NO':'no_NO'}
			);

			service.url = 'https://www.fedex.com/apps/fedextrack/?action=track&trackingnumber='+encodeURIComponent(no)+'&locale='+locale;
			var url = 'https://www.fedex.com/trackingCal/track';
			var options = new RequestOptions('fedex', childNo);
			options.service = service;
			options.locale = locale;
			var data = JSON.stringify({
				"TrackPackagesRequest":{
					"appType": "wtrk",
					"uniqueKey": "",
					"processingParameters" :{
						"anonymousTransaction": true,
						"clientId": "WTRK",
						"returnDetailedErrors": true,
						"returnLocalizedDateTime": false
					},
					"trackingInfoList":[{
						"trackNumberInfo":{
							"trackingNumber": no,
							"trackingQualifier": "",
							"trackingCarrier": ""
						}
					}]
				}
			});
			options.data = 'data='+data+'&action=trackpackages&locale='+locale+'&format=json&version=99';
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.gls-germany.com':

			var options = new RequestOptions('gls', childNo);
			var no = service.no.replace(/ /g,'');
			options.lang = preferredLanguage('en','de','fr','da');
			switch(options.lang) {
				case 'de':
					service.url = 'https://gls-group.eu/DE/de/paketverfolgung?txtAction=71000&match='+encodeURIComponent(no);
					break;
				case 'da':
					service.url = 'https://www.gls-group.eu/276-I-PORTAL-WEB/content/GLS/DK01/DA/5004.htm?txtAction=71000&txtRefNo='+encodeURIComponent(no);
					break;
				case 'fr':
					service.url = 'https://www.gls-group.eu/276-I-PORTAL-WEB/content/GLS/FR01/FR/5004.htm?txtAction=71000&txtRefNo='+encodeURIComponent(no);
					break;
				default:
					service.url = 'https://gls-group.eu/EU/en/parcel-tracking?txtAction=71000&match='+encodeURIComponent(no);
					break;
			}
			var url = 'https://gls-group.eu/app/service/open/rest/EU/'+options.lang+'/rstt001?match='+encodeURIComponent(no);
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.hlg.de':

			var options = new RequestOptions('hlg', childNo);
			var no = encodeURIComponent(service.no.replace(/[^A-Z0-9]/g,''));
			service.url = 'https://www.myhermes.de/empfangen/sendungsverfolgung/sendungsinformation/#'+no;
			var url = 'https://www.myhermes.de/services/tracking/shipments?search='+no;
			// &plz=00000
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.hongkongpost.com':

			service.url = 'https://www.hongkongpost.hk/en/mail_tracking/index.html';
			this.notSupported(isChild);
			this.updateItems();
			break;

		case 'www.japanpost.jp/ems':
		case 'www.japanpost.jp/m10':
		case 'www.japanpost.jp/reg':
		case 'www.japanpost.jp/yupack':

			var options = new RequestOptions('japan-post', childNo);
			options.lang = remapString(preferredLanguage('en','ja'), {'ja':'jp'});
			service.url = 'https://trackings.post.japanpost.jp/services/srv/search/direct?locale='+options.lang+
				'&reqCodeNo1='+encodeURIComponent(service.no);
			this.loadPage(service.url, this.getTracker, options);
			break;
	
		case 'www.laposte.fr':

			var options = new RequestOptions('la-poste', childNo);
			var no = service.no.replace(/[^A-Z0-9]/gi,'');
			options.lang = preferredLanguage('en','fr');
			if (options.lang == 'fr') {
				service.url = 'https://www.laposte.fr/outils/suivre-vos-envois?code='+no;
			} else {
				service.url = 'https://www.laposte.fr/outils/track-a-parcel?code='+no;
			}
			if (dashboardWidget) {
				this.notSupported(isChild);
				this.updateItems();
			} else {
				var url = 'https://api.laposte.fr/suivi/v1/'+no;
				this.loadPage(url, this.getJSON, options);
			}
			break;

		case 'www.lasership.com':
	
			var options = new RequestOptions('lasership', childNo);
			service.url = 'https://www.lasership.com/track/'+encodeURIComponent(service.no);
			var url = 'https://www.lasership.com/track/'+encodeURIComponent(service.no)+'/json';
			this.loadPage(url, this.getJSON, options);
			break;
	
		case 'www.ontrac.com':
	
			var options = new RequestOptions('ontrac', childNo);
			service.url = 'https://www.ontrac.com/trackingresults.asp?tracking_number='+encodeURIComponent(service.no);
			var url = 'https://www.ontrac.com/services/api/TrackingSummaryByTrackingNumbers/V1/?tracking='+encodeURIComponent(service.no);
			options.service = service;
			this.loadPage(url, this.getJSON, options);
			break;
	
		case 'www.parcelforce.com':

			var options = new RequestOptions('parcelforce', childNo);
			service.url = 'https://www.parcelforce.com/track-trace?trackNumber='+encodeURIComponent(service.no);
			var url = service.url+'&stop_mobi=yes';
			this.loadPage(url, this.getMultiple, options);
			break;
	
		case 'www.post.at':
	
			var options = new RequestOptions('post-at', childNo);
			var lang = preferredLanguage('en','de');
			service.url = (lang == 'de') ? 'https://www.post.at/sendungsverfolgung.php/details?pnum1='+encodeURIComponent(service.no) : 
				'https://www.post.at/en/track_trace.php/details?pnum1='+encodeURIComponent(service.no);
			this.loadPage(service.url, this.getMultiple, options);
			break;

		case 'www.postdanmark.dk': // www.postnord.dk
		case 'www.posten.se': // www.postnord.se

			var options = new RequestOptions('postnord', childNo);
			options.lang = preferredLanguage('en','sv','no','da','fi');
			if (service.from == 'www.posten.se') {
				var path = (options.lang == 'sv') ? 'online-verktyg/verktyg/sok/spara-brev-paket-och-pall' : 'en/online-tools/tools/track/track-and-trace';
				service.url = 'https://www.postnord.se/'+path+'#dynamicloading=true&shipmentid='+encodeURIComponent(service.no);
			} else {
				var path = (options.lang == 'da') ? 'track-trace' : 'en/track-and-trace';
				service.url = 'https://www.postnord.dk/'+path+'#dynamicloading=true&shipmentid='+encodeURIComponent(service.no);
			}
			var apiKey = extraInformationForService(service,'apiKey');
			if (apiKey) {
				var url = 'https://api2.postnord.com/rest/shipment/v1/trackandtrace/findByIdentifier.json?id='+encodeURIComponent(service.no)+'&locale='+options.lang+'&apikey='+encodeURIComponent(apiKey);
				this.loadPage(url, this.getJSON, options);
			} else {
				this.notSupported(isChild);
				this.updateItems();
			}
			break;

		case 'www.poste.it':
		case 'www.poste.it/posta1':
		case 'www.poste.it/pacco1':
		case 'www.poste.it/maxi':
		case 'www.poste.it/pacco':
		case 'www.poste.it/ems':

			var no = service.no.replace(/[^A-Za-z0-9]/g,'');
			service.url = 'https://www.poste.it/cerca/index.html#/risultati-spedizioni/'+no;
			var options = new RequestOptions('poste-it', childNo);
			var url = 'https://www.poste.it/online/dovequando/DQ-REST/ricercasemplice';
			options.data = JSON.stringify({"tipoRichiedente":"WEB","codiceSpedizione":no,"periodoRicerca":1});
			options.service = service;
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.posten.no':

			var no = service.no.replace(/[^A-Za-z0-9]/g,'');
			var options = new RequestOptions('posten-no', childNo);
			options.lang = preferredLanguage('en','no');
			service.url = 'https://sporing.posten.no/sporing/'+no+'?lang='+options.lang;
			var url = 'https://sporing.posten.no/tracking/api/fetch/'+no+'?lang='+options.lang;
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.postnl.nl':
		case 'www.tntpost.nl':

			var options = new RequestOptions('postnl', childNo);
			options.lang = preferredLanguage('nl','en','de','es','fr','it','zh');
			var postalCode = service.extra;
			var countryCode = 'NL';
			if (postalCode) {
				var matches = null;
				if (matches = postalCode.match(/(.*)[\/, -]+([A-Z]{2})$/i)) {
					postalCode = trim(matches[1]);
					countryCode = trim(matches[2]);
				} else if (matches = postalCode.match(/^([A-Z]{2})[\/, -]+(.*)/i)) {
					postalCode = trim(matches[2]);
					countryCode = trim(matches[1]);
				} else if (matches = postalCode.match(/^([A-Z]{2})$/i)) {
					postalCode = '';
					countryCode = trim(matches[1]);
				}
			}
			service.url = 'https://www.internationalparceltracking.com/Main.aspx#/track/'+encodeURIComponent(service.no)+'/'+countryCode;
			var url = 'https://www.internationalparceltracking.com/api/shipment?language='+options.lang+'&barcode='+encodeURIComponent(service.no)+'&country='+countryCode;
			if (postalCode) {
				service.url += '/'+encodeURIComponent(postalCode);
				url += '&postalCode='+encodeURIComponent(postalCode);
			}
			this.loadPage(url, this.getJSON, options);
			break;
	
		case 'www.purolator.com':
	
			var options = new RequestOptions('purolator', childNo);
			options.lang = preferredLanguage('en','fr');
			service.url = 'https://www.purolator.com/'+options.lang+'/ship-track/tracking-summary.page?pin='+encodeURIComponent(service.no);
			this.loadPage(service.url, this.getMultiple, options);
			break;

		case 'www.royalmail.com':

			service.url = 'https://www.royalmail.com/track-your-item#/tracking-results/'+encodeURIComponent(service.no);
			this.notSupported(isChild);
			this.updateItems();
			break;

		case 'www.sagawa-exp.co.jp':
	
			service.url = 'https://k2k.sagawa-exp.co.jp/p/web/okurijosearch.do?okurijoNo='+encodeURIComponent(service.no);
			var options = new RequestOptions('sagawa', childNo);
			this.loadPage(service.url, this.getTracker, options);
			break;
	
		case 'www.post.ch':

			var lang = preferredLanguage();
			service.url = 'https://service.post.ch/EasyTrack/submitParcelData.do?lang='+lang+'&formattedParcelCodes='+encodeURIComponent(service.no);
			var options = new RequestOptions('swiss-post', childNo);
			this.loadPage(service.url, this.getTracker, options);
			break;
	
		case 'www.tnt.com':
		case 'www.tnt.com/ref':
	
			var mode = 'CON';
			var key = 'con';
			if (service.from == 'www.tnt.com/ref') {
				mode = 'REF';
				key = 'ref';
			}
			service.url = 'https://www.tnt.com/express/en_us/site/tracking.html?searchType='+
				mode+'&cons='+encodeURIComponent(service.no);
			var url = 'https://www.tnt.com/api/v3/shipment?searchType='+
				mode+'&'+key+'='+encodeURIComponent(service.no)+'&locale=en_US&channel=OPENTRACK';
			var options = new RequestOptions('tnt', childNo);
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.tntexpress.com.au':

			var no = encodeURIComponent(service.no);
			service.url = 'https://www.tntexpress.com.au/interaction/trackntrace.aspx?con='+no;
			var url = (commonFilesVersion < 211) ? 
				'http://www.tntexpress.com.au/interaction/ASPs/Trackcon_tntau.asp?id=TRACKCON.ASP&con='+no : 
				'https://www.tntexpress.com.au/interaction/ASPs/Trackcon_tntau.asp?id=TRACKCON.ASP&con='+no;
			var options = new RequestOptions('tnt-express', childNo);
			options.service = service;
			this.loadPage(url, this.getTracker, options);
			break;

		case 'www.ups.com':

			var no = encodeURIComponent(service.no.replace(/ /g,''));
			var locale = remapString(
				preferredLocale('en_US','de_DE','fr_FR','ja_JP','it_IT','sv_SE','da_DK','nl_NL','no_NO','nb_NO'),
				{'nb_NO':'no_NO'}
			);
			var options = new RequestOptions('ups', childNo);
			options.service = service;
			options.locale = locale;
			service.url = 'https://www.ups.com/track?loc='+locale+'&tracknum='+no+'&requester=WT';
			var url = 'https://www.ups.com/track/api/Track/GetStatus?loc='+locale;
			options.data = JSON.stringify({
				"Locale": locale,
				"TrackingNumber": [no]
			});
			this.loadPage(url, this.getJSON, options);
			break;

		case 'www.ups-mi.net':
	
			service.url = 'https://www.ups-mi.net/packageID/packageid.aspx?pid='+encodeURIComponent(service.no);
			var options = new RequestOptions('ups-mi', childNo);
			this.loadPage(service.url, this.getTracker, options);
			break;

		case 'www.usps.com':

			var no = (service.no.toUpperCase()).replace(/[^A-Z0-9]/g,'');
			service.url = 'https://tools.usps.com/go/TrackConfirmAction.action?tLabels='+encodeURIComponent(no);
			var apiUserID = extraInformationForService(service, 'apiUserID');

			if (apiUserID) {
				var options = new RequestOptions('usps', childNo);
				var clientIPAddress = extraInformationForService(service,'clientIPAddress');
				var extraData = (apiUserID && clientIPAddress) ? '<Revision>1</Revision><ClientIp>'+clientIPAddress+'</ClientIp><SourceId>Junecloud</SourceId>' : '';
				var url = 'https://secure.shippingapis.com/ShippingAPI.dll';
				options.data = 'API=TrackV2&XML='+encodeURIComponent('<TrackFieldRequest USERID="'+apiUserID+'">'+extraData+'<TrackID ID="'+no+'"></TrackID></TrackFieldRequest>');
				this.loadPage(url, this.getTracker, options);
			} else {
				var options = new RequestOptions('usps-legacy', childNo);
				this.loadPage(service.url, this.getTracker, options);
			}
			break;
	
		case 'www.kuronekoyamato.co.jp':
	
			var url = 'http://toi.kuronekoyamato.co.jp/cgi-bin/tneko';
			service.url = 'http://jizen.kuronekoyamato.co.jp/jizen/servlet/crjz.b.NQ0010?id='+encodeURIComponent(service.no.replace(/-/g,''));
			var options = new RequestOptions('yamato', childNo);
			options.data = 'number00=1&number01='+encodeURIComponent(service.no);
			options.service = service;
			this.loadPage(url, this.getMultiple, options);
			break;

		case 'www.yodel.co.uk':

			var options = new RequestOptions('yodel', childNo);
			var no = service.no.toUpperCase().replace(/ /g,'');
			var extra = service.extra.toUpperCase().replace(/ /g,'');
			service.url = 'https://www.yodel.co.uk/tracking/'+encodeURIComponent(no)+'/'+encodeURIComponent(extra);
			this.loadPage(service.url, this.getTracker, options);
			break;

		case 'other':

			this.url = this.extra;
			this.updatePlaceholder();
			this.updateItems();
			break;

		case 'checkout.google.com':

			this.notSupported(isChild);
			this.updateItems();
			break;

		default:

			this.notSupported(isChild, true);
			this.updateItems();
			break;

	}

}

Delivery.prototype.getAmazon = function (request) {

	var delivery = this;
	var sections = false;
	var items = false;
	var result = false;
	var trackURL = false;
	var x = 0;
	var y = 0;
	var z = 0;
	var i = 0;

	var output = '';
	if (request.responseText) output = stripWhiteSpace(request.responseText);
	var errorResult = output.match(/<div class="a-box a-alert a-alert-(?:info|warning)[^"]*"> ?<div class="a-box-inner a-alert-container"> ?<h4 class="a-alert-heading">([^<]*)/i);

	if (output && !errorResult) {

		var langFrom = getLanguageFrom(delivery.from, output);
		var options = new ParseOptions(langFrom);

		// If it's the page showing all orders, try to find the order there
		if (output.match(/<div id="yourOrders"> ?<div id="yourOrdersContent">/i)) {
			var orders = overlappingMatches(output,'(?:<div class="a-box-group a-spacing-base order">)(.*?)(?:<div class="a-box-group a-spacing-base order">|<div class="a-row"> <div class="a-text-center pagination-full">|$)');
			for (var orderx in orders) {
				var order = orders[orderx];
				var orderno = order.match(/<span class="a-color-secondary value"> ?([0-9]{3}-[0-9]{7}-[0-9]{7}) ?<\/span>/i);
				if (orderno && orderno[1]) {
					if (orderno[1] == delivery.no) {
						output = order; 
						break;
					}
				}
			}
		}

		sections = overlappingMatches(output,'<div[^>]*class="a-box [^"]*shipment[^"]*">.*?(<div[^>]*class="a-box [^"]*shipment[^"]*">|<script type="text\/javascript">|$)');

		for (y in sections) {

			var secDeliveredDate = false;
			var status = '';

			// Sometimes there are subsections with different dates, especially for pre-orders
			var subSections = overlappingMatches(sections[y],'<div class="a-row"> ?(?:<div class="a-section a-spacing-base [^"]*"> ?(<div>)? ?<span[^>]*>|<div> ?<span[^>]*>)(.*?)(<div class="a-row"> ?(?:<div class="a-section a-spacing-base [^"]*"> ?(<div>)? ?<span[^>]*>|<div> ?<span[^>]*>)|$)');
			if (!subSections) subSections = [sections[y]];
			for (z in subSections) {

				// Get the status of this section, which may include the delivery date
				result = sections[y].match(/(?:<span class="a-size-medium[^"]*">|<h3>)([^<]*)/i); // h3 not used anymore?
				if (result) {
					status = stripHTML(result[1]);
					if (result = status.match(/^ ?(?:Delivered|Zugestellt|Entregado|Livré|Consegnato|配達を完了しました|配達完了):?(.*?)$/i)) {
						status = 'Complete';
						if (result[1]) {
							secDeliveredDate = parseDate(result[1], options);
						}
					} else {
						result = status.match(/^ ?(?:Arriving|Expected(?: by)?|Now expected|Zustellung|Erwartete Zustellung|Voraussichtlich|Voraussichtliche Lieferung|Entrega(?: el)?|Entrega prevista|Llegada(?: el| antes del)?|Livraison(?: prévue| estimée)?(?: le)?|Arriverà|In consegna):? (.*)/i);
						if (!result) result = status.match(/^ ?([0-9]+月[0-9]+日|[0-9\/]*)[^<]*に到着予定/i);
						if (result && result[1]) {
							var deliveredRaw = endDateInRange(result[1]);
							secDeliveredDate = parseDate(deliveredRaw, options, true);
							status = '';
						}
					}
				}

				var item = new DeliveryPackage();
				item.status = status;

				if (secDeliveredDate) item.deliveredDate = secDeliveredDate;

				// Get the estimated delivery date: "Voraussichtlicher Liefertermin:", "Delivery estimate:", "Guaranteed delivery by:"
				// Or the actual delivered date: "Delivered on:"
				// And/or the ship date: "Shipping estimate:"
				result = subSections[z].match(/<div[^>]*> ?<span class="a-color-success">([^<]*)(?:<span class="a-text-bold">([^<]*))?(?: ?<\/span>| ?<\/div>)*(?:<div[^>]*> ?<span class="a-color-success">([^<]*)(?:<span class="a-text-bold">([^<]*))?(?: ?<\/span>| ?<\/div>)*)?/i);
				if (!result) result = subSections[z].match(/<div class="a-row"> ?(?:<div class="a-section a-spacing-base a-spacing-top-large"> ?)?<div> ?<span> ?([a-z\s]*): ?([^<]*)/i);
				if (result) {
					var label = trim(result[1]);
					var date = null;
					if (result[2]) date = parseDate(endDateInRange(result[2]), options, true);
					if (date) {
						if (result[4]) {
							item.shippedDate = date;
							item.deliveredDate = parseDate(endDateInRange(result[4]), options, true);
						} else {
							if (label && label.match(/^(Shipping estimate)/i)) {
								item.shippedDate = date;
							} else item.deliveredDate = date;
						}
					}
				}

				// Get the tracking link for shipped items
				if (result = subSections[z].match(/<a href="(https:\/\/[^\/]*)?(\/(?:progress-tracker\/package|gp\/your-account\/ship-track|gp\/css\/shiptrack)[\/?%][^"]+)"[^>]*>/i)) {
					trackURL = result[2].replace(/&amp;/g,'&');
					if (result[1]) {
						trackURL = result[1]+trackURL;
					} else {
						trackURL = 'https://'+delivery.domain+trackURL;
					}
					var options = new RequestOptions('amazon-track', x);
					this.loadPage(trackURL, this.getTracker, options);
				}

				// Get the name of each item
				var items = subSections[z].match(/(<div class="a-row">|<span class="a-size-base">) ?<a class="a-link-normal"[^>]*>([^<]*)/gi);
				var firstItem = true;
				for (i in items) {
					// Get the name of the item
					result = items[i].match(/<a class="a-link-normal"[^>]*>([^<]*)/i);
					if (result && result[1]) {
						result = trim(result[1]);
						result = result.replace('&iuml;&iquest;&frac12;',' '); // Clean up encoding errors
						if (firstItem) {
							firstItem = false;
							item.label = result;
						} else {
							item.label += ", "+result;
						}
					}
				}

				if (x == 0) delivery.items = [];
				delivery.items[x] = item;
				x++;

			}

		}

		delivery.count = x;
		if (!delivery.hasItems()) {
			delivery.count = 0;
		} else {
			delivery.updatePackageNames();
		}

	}

	if (delivery.count == 0) {

		// Check for an error message or sign in form
		var error = DeliveryError.notFound;
		var needsSignIn = output.match(/(?:<form[^>]*name=.?signIn.?[^>]*action="(?:[^"]*)[^>]*>|<div class="[^"]* auth-workflow">|<form id="auth-mfa-form"[^>]*>)/i);

		result = output.match(/<div class="a-alert-content"> ?<ul[^>]*> ?<li> ?<span[^>]*>([^<]*)<\/span> ?<\/li> ?<\/ul> ?<\/div>/i);
		if (!result) result = errorResult; // Cancelled, not found, problem loading
		if (!result) result = output.match(/(?:<div id="message_error"[^>]*>|<div class="message error"[^>]*>).*?(?:<h6>[^<]*<\/h6>)? ?<p>(.*?)<\/p> ?<\/div>/i);
		if (result) result = result[1];

		if (result || needsSignIn) {
			if (needsSignIn) {
				error = DeliveryError.signIn;
				var message = 'getAmazon requires sign in: '+needsSignIn[0];
				if (result) message += ' error: '+result;
				logMessage(1,message);
				manualSignIn[delivery.from+'-'+delivery.extra] = true;
			}
			this.updateFailed(error, result);
		} else {
			// Note that this can happen for both server errors and incorrect order numbers:
			// <h4 class="a-alert-heading">There's a problem loading this order
			var serverError = (request.status == 500 || output.match(/(<title>500 Service Unavailable Error|<body> ?<div style="width: 90%; margin-left: 5%"> ?<h2>)/));
			if (serverError) error = DeliveryError.server;
			logRequest(1,request);
			this.updateFailed(error, null);
		}

	}

	this.reqComplete++;
	this.updateItems();

}

Delivery.prototype.getAppleJSON = function (request) {

	var delivery = this;
	var pattern = /<script id="init_data" type="application\/json">(.*?)<\/script>/;
	var object = parseJSONInHTML(request.responseText, pattern);
	var orderItems = (object && object.orderDetail) ? object.orderDetail.orderItems : null;
	var orderItemList = (orderItems) ? orderItems.c : null;

	if (orderItems && orderItemList) {

		for (var x in orderItemList) {

			var itemID = orderItemList[x];
			var orderItem = orderItems[itemID];
			var detailItem = orderItem.orderItemDetails.d;
			var statusItem = orderItem.orderItemStatusTracker.d;

			var name = detailItem.productName;
			if (name == null && itemID.indexOf('giveBackOrderItem') === 0) name = 'Apple GiveBack';

			var detailStatus = detailItem.deliveryDate;
			var status = getAppleState(statusItem.currentStatus, request.responseText) || detailStatus || '';
			var dates = getAppleDates(detailStatus);
			var options = new ParseOptions(delivery.from);
			var futureDate = (status != 'Complete');
			var shippedDate = parseDate(dates[0], options, futureDate);
			var deliveredDate = parseDate(dates[1], options, futureDate);
			var urls = detailItem.trackingURLMap || [null];

			for (var i in urls) {

				var url = urls[i];
				var item = new DeliveryPackage();
				item.label = name;
				item.status = status;
				item.trackURL = url;
				item.shippedDate = shippedDate;
				item.deliveredDate = deliveredDate;
				delivery.items[x] = item;

				if (url) {
					var service = serviceFromURL(url);
					if (service) delivery.startUpdate(service, x);
				}

				x++;

			}

		}
		delivery.count = delivery.items.length;
		delivery.updatePackageNames();

	} else {
		logRequest(1,request);
		var error = (object) ? DeliveryError.notFound : DeliveryError.server;
		this.updateFailed(error, null);
	}

	this.reqComplete++;
	this.updateItems();

}

function getAppleState (status, html) {
	switch (status) {
		case 'DELIVERED': return 'Complete';
		case 'DELIVERED_TO_YOU': return 'Complete';
		default:
			var exp = new RegExp('"stateText" ?: ?({[\\s\\S]*?})');
			var result = html.match(exp);
			if (!result) return null;
			var object = {};
			try {
				object = JSON.parse(result[1]);
			} catch (err) {
				return null;
			}
			return object[status];
	}
	return null;
}

Delivery.prototype.getApple = function (request) {

	var delivery = this;
	var result = false;
	var checkForError = false;
	var x = 0;

	if (request.status == 200 && request.responseText) {

		var output = stripWhiteSpace(request.responseText);
		output = output.replace(/[\u2028]+/g,' '); // For iOS 4, in 4.7 and earlier

		var trackInfo = getAppleTrackInfo(output, delivery.no);
		var sections = output.match(/<div class="shaded-box">(.*?)<\/div> ?<\/li> ?(?:<!-- \/\.product -->)? ?<\/ul> ?<\/div> ?<\/div> ?<\/div>/gi);

		for (var y in sections) {

			var secOutput = sections[y];
			secOutput = secOutput.replace(/(\s*<!--(.*?)-->\s*)+/g,' ');

			var secInfo = false;
			var secShippedRaw = false;
			var secDeliveredRaw = false;
			var status = false;
			var trackURL = false;
			var item = new DeliveryPackage();

			// Get the status of this section, otherwise assume it's the same as the previous item
			result = secOutput.match(/<h3[^>]*> ?<span[^>]*>([^<]*)/i);
			if (result && result[1]) status = trim(result[1]);
			if (status) {

				// Ignore dates for cancelled or picked up items
				// Not sure if "Replaced with New Product" is still used
				if (!status.match(/^(Cancelled|Canceled|Replaced with New Product|Returns Completed|Picked Up)$/i)) {

					result = secOutput.match(/<span class="h5">( ?on )?([^<|]*)((<\/span> ?)?<span[^>]*>( ?<br\/> ?)?([^<]*))?/i);
					if (result && result[2]) {
						result = getAppleDates(result[0]);
						if (result[0]) secShippedRaw = result[0];
						secDeliveredRaw = result[1];
					}

					// Get the tracking number
					result = secOutput.match(/<(?:a|button)[^>]*id="[A-Z0-9]+(-tik)?-track-delivery-link-([A-Z0-9]+)"[^>]*>/i);
					if (result && result[2]) {
						var id = (result[1]) ? 'tikgroup' : 'group';
						id += '-'+result[2];
						trackURL = trackInfo[id];
					}

					if (status.match(/^Delivered|配送済み|已送達$/)) {
						status = 'Complete';
					}

				}

				// Get each item in the section
				var items = secOutput.match(/(<li class="delivery-group">|<div class="media-block product-item">)(.*?)<\/div> ?<\/li>/gi);
				for (var z in items) {

					item.status = status;
					item.details = {};
					// item.details.status = status;
					item.trackURL = trackURL;

					// Get the name of the item
					result = items[z].match(/<h4 class="strong name">([^<]*)/i);
					if (result && result[1]) item.label = trim(result[1]);

					var dateInfo = cleanSpaces(items[z]);
					result = getAppleDates(dateInfo);
					item.shippedRaw = (result[0]) ? result[0] : secShippedRaw;
					item.deliveredRaw = (result[1]) ? result[1] : secDeliveredRaw;

					if (z == 0) {
						if (!secShippedRaw) secShippedRaw = item.shippedRaw;
						if (!secDeliveredRaw) secDeliveredRaw = item.deliveredRaw;
					}

					var lastItem = false;
					if (x == 0) {
						delivery.items = [];
					} else lastItem = delivery.items[x-1];

					if (lastItem && 
						item.status == lastItem.status && 
						item.shippedRaw == lastItem.shippedRaw && 
						item.deliveredRaw == lastItem.deliveredRaw &&
						item.trackURL == lastItem.trackURL) {

						lastItem.label += ', '+item.label;

					} else {

						delivery.items[x] = item;	
						if (trackURL) {
							var options = new RequestOptions(null, x);
							var result = trackURL.match(/https?:\/\/([^\/]*)/i);
							if (result && result[1]) {
								if (result[1].match(/^(secure[0-9]*\.)?store\.apple\.com$/)) {
									this.loadPage(trackURL, this.getAppleDetails, options);
								} else {
									logMessage(1,'Unsupported Apple details URL: '+trackURL);
								}
							}
						}
	
						item = new DeliveryPackage();
						x++;

					}

				}

			}

		}

		delivery.count = delivery.items.length;
		if (!delivery.hasItems()) {
			delivery.count = 0;
		} else {
			delivery.updatePackageNames();
		}

		if (delivery.items.length > 0) {
			for (var z in delivery.items) {
				var item = delivery.items[z];
				var options = new ParseOptions(delivery.from);
				var futureDate = (item.status != 'Complete');
				delivery.items[z].shippedDate = parseDate(item.shippedRaw, options, futureDate);
				delivery.items[z].deliveredDate = parseDate(item.deliveredRaw, options, futureDate);
			}
		}

		if (x == 0) checkForError = true;

	} else {
		checkForError = true;
	}

	if (checkForError) {
		var error = DeliveryError.notFound;
		logRequest(1,request);
		if (request.responseText && (output = stripWhiteSpace(request.responseText)) && output.match(/(We'll be back soon|Apple's Online and phone Order Status services are temporarily unavailable due to a scheduled upgrade to our systems|We are currently updating our systems)/i)) {
			error = DeliveryError.server;
		} else {
			delivery.count = 0;
		}
		this.updateFailed(error, null);
	}

	this.reqComplete++;
	this.updateItems();

}

function getAppleTrackInfo (html, no) {
	var result = html.match(/window.OrderStatusData ?= ?({.*?});/);
	if (!result) return null;
	var actions = null;
	try {
		var object = JSON.parse(result[1]);
		actions = object.replace.orders[no]['_actions'];
	} catch (err) {
		logMessage(1,err);
	}
	if (!actions) return null;
	var urls = [];
	var groups = [];
	groups['group'] = actions.group;
	groups['tikgroup'] = actions.tikgroup;
	for (var a in groups) {
		if (!groups.hasOwnProperty(a)) continue;
		var group = groups[a];
		for (var b in group) {
			if (!group.hasOwnProperty(b)) continue;
			var item = group[b];
			if (!item) continue;
			for (var c in item) {
				if (!item.hasOwnProperty(c)) continue;
				var status = item[c];
				var track = status['TRACK_SHIPMENT'];
				if (!track) continue;
				urls[a+'-'+b] = track.url;	
			}
		}
	}
	return urls;
}

function getAppleDates ( html ) {

	if (!html) return [];

	var ship = '';
	var deliver = '';

	// New: Arrives|Delivered
	var result = html.match(/(?:Arrives|Delivered|Estimated Delivery|Delivers on|Delivers by|Delivers between|Delivers|Delivered on|配送日|お届け予定日|運送日期|送達時間)[:：]? ?([^<]*)/);
	if (result && result[1]) {
		deliver = result[1].replace(/(via|by|：)[^:<]+$/i,'');
		var dates = datesInRange(deliver);
		deliver = dates[1];
	} else {
		var result = html.match(/^(Out for Delivery)$/);
		if (result && result[1]) deliver = 'Today';
	}

	result = html.match(/(?:Shipped|Available to ship|Estimated Shipping|Ships by|Estimated Ship|Ships|出荷日|已上市)[:：]? ?([^<]*)/);
	if (result && result[1]) {
		// Fix ugly stuff like Ships: Ships by October 12th
		ship = result[1].replace(/(?:Shipped|Available to ship|Estimated Shipping|Ships by|Estimated Ship|Ships|出荷日|已上市)[:：]?/,'');
		var dates = datesInRange(ship);
		ship = dates[1];
		result = ship.match(/(?:Arrives|Delivered|Estimated Delivery|Delivers on|Delivers by|Delivers between|Delivers|Delivered on|配送日|お届け予定日|運送日期|送達時間)[:：]? ?([^<]*)/);
		if (result && result[1]) {
			if (!deliver) deliver = result[1];
			ship = ship.replace(result[0],'');
		}
		ship = (trim(ship)) ? ship : '';
	}

	if (!ship && !deliver) {
		html = html.replace(/<select[^>]*>.*?<\/select>/i,'');
		result = html.match(/(?:Pick up):? ?([^<]*)/);
		if (result && result[1]) {
			deliver = result[0];
		}
	}

	return [ship, deliver];

}

function serviceFromURL (url) {

	var result = url.match(/https?:\/\/([^\/]*)/i);
	if (result && result[1]) {

		var domain = result[1];
		var service = null;

		switch (domain) {
			case 'www.dhl.com':
			case 'mrt2.ap.dhl.com':
				var data = getData(url);
				var no = data['AWB'];
				if (no) service = new Service('www.dhl.com', no);
				break;
			case 'www.fedex.com':
				var data = getData(url);
				var no = data['tracknumbers'];
				if (no) service = new Service('www.fedex.com', no);
				break;
			case 'toidirect.kuronekoyamato.co.jp':
				var data = getData(url);
				var no = data['ctn'];
				if (no) service = new Service('www.kuronekoyamato.co.jp', no);
				break;
			case 'www.ontrac.com':
				var data = getData(url);
				var no = getDataValue(data,'tracking','tracking_number');
				if (no) service = new Service('www.ontrac.com', no);
				break;
			case 'www.ups.com':
			case 'wwwapps.ups.com':
				var data = getData(url);
				var no = getDataValue(data,'InquiryNumber1','tracknum','trackNums',
					'inquiry1','trackingNumber','InquiryNumber');
				if (no) service = new Service('www.ups.com', no);
				break;
			case 'www.tnt.com':
				break;
			default:
				break;
		}

		return service;

	}
	return null;

}

Delivery.prototype.getAppleDetails = function (request) {

	var delivery = this;
	var childNo = request.options.childNo;

	if (request.responseText) {

		var output = stripWhiteSpace(request.responseText);
		var result;
		if (result = output.match(/<a[^>]*data-action=\\"tracking-number\\"[^>]*href=\\"(.*?)\\"[^>]*>/i)) {

			var url = result[1];
			var service = serviceFromURL(url);
			if (service) {
				this.startUpdate(service,childNo);
			} else {
				this.reqCount++;
				request.options.site = 'apple-inline';
				this.getTracker(request);
			}

		} else {
			this.reqCount++;
			request.options.site = 'apple-inline';
			this.getTracker(request);
		}

	} else {
		logRequest(1,request);
		this.updateFailed(DeliveryError.server, null);
	}

	this.reqComplete++;
	this.updateItems();

}

Delivery.prototype.getFedExMultiple = function (request) {

	var delivery = this;
	var result = false;

	var output = trim(request.responseText);
	if ((request.status == 200 || request.status == 0) && output) {

		var object = {};
		try {
			output = output.replace(/\\x/g,'%');
			object = JSON.parse(output);
		} catch (err) {
			logMessage(1,err);
			logRequest(1,request);
			success = false;
		}

		if (object) object = object.AssociatedShipmentsResponse;
		if (object && object.successful && object.associatedShipmentList) {
			for (var x in object.associatedShipmentList) {

				var anObject = object.associatedShipmentList[x];
				if (anObject.isSuccessful) {
					var trackingNbr = decodeURIComponent(anObject.trackingNbr);
					if (delivery.items && delivery.items[0] && (
						decodeURIComponent(anObject.trackingQualifier) == delivery.items[0].matchNo ||
						trackingNbr == delivery.no.replace(/\s+/g,''))) {

						delivery.items[0].label = trackingNbr;


					} else {

						var item = new DeliveryPackage();
						item.label = trackingNbr;

						var status = decodeURIComponent(anObject.keyStatus);
						if (anObject.delivered) status = 'Complete';
						if (status) item.status = status;
						var futureDate = (status != 'Complete');

						var ship = decodeURIComponent(anObject.shipDt);
						var deliver = (anObject.actDeliveryDt) ? decodeURIComponent(anObject.actDeliveryDt) : 
							decodeURIComponent(anObject.estDeliveryDt);
						if (deliver == 'Pending') deliver = false;
						var options = new ParseOptions(delivery.from);
						if (ship) item.shippedDate = parseDate(ship, options);
						if (deliver) item.deliveredDate = parseDate(deliver, options, futureDate);

						var count = delivery.items.push(item);
						var service = new Service('www.fedex.com', trackingNbr);
						if (service) this.startUpdate(service, count-1);

					}
				}

			}
		}
		delivery.count = delivery.items.length;
		delivery.updatePackageNames();

	}

	this.reqComplete++;
	this.updateItems();

}

Delivery.prototype.getJSON = function (request) {

	var delivery = this;
	var result = false;
	var site = request.options.site;
	var childNo = request.options.childNo;
	var isChild = request.options.isChild;
	var ignoreStatus = ['aramex', 'gls', 'la-poste', 'lasership', 
		'posten-no', 'postnord', 'tnt'];

	var output = trim(request.responseText);

	// If it's wrapped in a function, or HTML, unwrap it
	if (site == 'dhl-germany') {
		var result = output.match(/initialState: JSON\.parse\("(.*?)"\)/);
		if (result) output = result[1].replace(/\\\"/g,'"');
	} else if (site == 'dpd') {
		output = output.replace(/^[a-z0-9_]*\(((.|[\r\n])*)\);?$/,'$1');
	}

	if ((request.status == 200 || request.status == 0 || ignoreStatus.indexOf(site) != -1) && output) {

		var object = {};
		var from = (request.options.service) ? request.options.service.from : delivery.from;
		var parseOptions = new ParseOptions(from);
		var deliveredPtn = '^Delivered$';
		var connect = true;
		var success = true;
		var items = [];
		var errorText;

		try {
			if (site == 'fedex') output = output.replace(/\\x/g,'%');
			object = JSON.parse(output);
		} catch (err) {
			logMessage(1,err);
			logRequest(1,request);
			success = false;
		}

		if (site == 'aramex') {

			deliveredPtn = '100%';

			object = object[0];
			if (object && object.found == true) {
				var item = new DeliveryPackage();
				item.status = object.currentStatus;
				item.shippedRaw = object.dateShipped;
				item.deliveredRaw = object.deliveredDate;
				if (!item.deliveredRaw && object.transitMsg) {
					var result = object.transitMsg.match(/delivered by ([0-9\/]+)/i);
					if (result) item.deliveredRaw = result[1];
				}
				item.deliveredStatus = object.trackingProgress;

				var oEvents = object.packageHistory;
				var history = [];
				if (oEvents) {
					for (var e = 0; e < oEvents.length; e++) {
						var oEvent = oEvents[e];
						var event = new DeliveryEvent();
						event.dateRaw = oEvent.actionDate;
						event.status = oEvent.activity;
						event.location = oEvent.location;					
						history.push(event);
					}
				}
				item.history = history;
				items.push(item);
			} else {
				success = false;
			}

		} else if (site == 'canada-post') {

			var lang = request.options.lang;

			if (object && object.status) {

				var item = new DeliveryPackage();
				item.status = object.status;
				if (object.actualDlvryDate) {
					item.deliveredRaw = object.actualDlvryDate;
				} else if (object.expectedDlvryDateTime) {
					item.deliveredRaw = object.expectedDlvryDateTime.revisedDate || object.expectedDlvryDateTime.dlvryDate;
				}
				if (object.delivered == 'true') item.deliveredStatus = 'Delivered';

				var oEvents = object.events;
				var history = [];
				if (oEvents) {
					for (var e = 0; e < oEvents.length; e++) {
						var oEvent = oEvents[e];
						var event = new DeliveryEvent();
						event.dateRaw = oEvent.datetime.date;
						event.status = (lang == 'fr') ? oEvent.descFr : oEvent.descEn;
						if (oEvent.locationAddr) {
							var parts = oEvent.locationAddr;
							var country = (lang == 'fr') ? parts.countryNmFr : parts.countryNmEn;
							var location = locationFromParts(parts.city,parts.regionCd,null,country);
							if (location) event.location = location;
						}
						history.push(event);
					}
				}
				item.history = history;
				items.push(item);

			} else {
				success = false;
				if (object && object.error) {
					errorText = (lang == 'fr') ? object.error.descFr : object.error.descEn;
				}
			}

		} else if (site == 'dhl') {

			deliveredPtn = '^delivered$';

			if (object && object.results) {

				var oItems = object.results;
				for (var i = 0; i < oItems.length; i++) {

					var oItem = oItems[i];
					var item = new DeliveryPackage();

					item.status = oItem.description;
					// if (oItem.signature) item.deliveredRaw = oItem.signature.description;
					if (oItem.edd) {
						item.deliveredRaw = oItem.edd.date || oItem.edd.product;
					}
					if (oItem.delivery) item.deliveredStatus = oItem.delivery.status;
					if (oItem.destination) item.deliveredLocation = oItem.destination.value;

					var oEvents = oItem.checkpoints;
					var history = [];
					if (oEvents) {
						for (var e = 0; e < oEvents.length; e++) {
							var oEvent = oEvents[e];
							var event = new DeliveryEvent();
							event.dateRaw = oEvent.date;
							event.status = oEvent.description;
							event.location = oEvent.location;
							history.push(event);
						}
					}
					item.history = history;
					item.label = decodeURIComponent(oItem.id);
					items.push(item);

				}

			} else {
				success = false;
				if (object && object.errors) {
					object = object.errors[0];
					if (object) errorText = object.message;
				}
			}

		} else if (site == 'dhl-germany') {

			if (object && object.sendungen) {
				var objects = object.sendungen;
				object = objects[0];
				for (var x = 0; x < objects.length; x++) {
					if (objects[x].id == delivery.no) object = objects[x];
				}
			}

			if (object && object.sendungsdetails && !object.sendungNichtGefunden) {

				var oItem = object.sendungsdetails;
				var item = new DeliveryPackage();

				var oDetail = oItem.sendungsverlauf || {};
				item.status = oDetail.aktuellerStatus;
				if (oItem.istZugestellt) {
					item.deliveredStatus = 'Delivered';
				}
				if (oItem.zustellung) {
					item.deliveredRaw = oItem.zustellung.zustellzeitfensterBis || 
						oItem.zustellung.zustellzeitfensterVon;
				}

				var oEvents = oDetail.events;
				var history = [];
				if (oEvents) {
					for (var e = 0; e < oEvents.length; e++) {
						var oEvent = oEvents[e];
						var event = new DeliveryEvent();
						event.dateRaw = oEvent.datum;
						event.status = oEvent.status;
						event.location = oEvent.ort;
						history.unshift(event);
					}
				}
				item.history = history;
				items.push(item);

			} else {
				success = false;
				if (object && object.sendungNichtGefunden) {
					// No data: sendungNichtGefunden.keineDatenVerfuegbar = true
					// Invalid no.: sendungNichtGefunden.sendungsnummerUngueltig = true
				} else {
					connect = false;
				}
			}

		} else if (site == 'dpd') {

			deliveredPtn = '^(DELIVERED|PICKUP_FROM_PARCELSHOP_BY_CONSIGNEE)$';

			if (object.parcellifecycleResponse && object.parcellifecycleResponse.parcelLifeCycleData) {

				var oItem = object.parcellifecycleResponse.parcelLifeCycleData;
				var item = new DeliveryPackage();

				var oInfos = oItem.statusInfo;
				if (oInfos) {
					for (var x = 0; x < oInfos.length; x++) {
						var oInfo = oInfos[x];
						if (oInfo.isCurrentStatus) {
							item.status = oInfo.label;
							item.deliveredStatus = oInfo.status;
						}
					}
				}

				var oEvents = (oItem.scanInfo) ? oItem.scanInfo.scan : null;
				var history = [];
				if (oEvents) {
					for (var e = 0; e < oEvents.length; e++) {
						var oEvent = oEvents[e];
						var event = new DeliveryEvent();
						event.dateRaw = oEvent.date;
						event.location = (oEvent.scanData) ? oEvent.scanData.location : null;
						if (oEvent.scanDescription && oEvent.scanDescription.content) {
							event.status = oEvent.scanDescription.content[0];
						}
						history.unshift(event);
					}
				}
				item.history = history;
				items.push(item);

			} else {
				success = false;
				if (object.serviceMessage) {
					object = object.serviceMessage;
					if (object.validationError) {
						errorText = object.validationError.message;
					}
				}
			}

		} else if (site == 'fedex') {

			if (object) object = object.TrackPackagesResponse;
			if (object && object.successful && object.packageList) {
				var oItems = object.packageList;
				for (var i = 0; i < oItems.length; i++) {
					var oItem = oItems[i];
					if (oItem.isSuccessful) {

						var item = new DeliveryPackage();
						item.status = oItem.mainStatus;
						if (oItem.isDelivered) item.deliveredStatus = 'Delivered';
						item.shippedRaw = decodeURIComponent(oItem.shipDt);
						item.deliveredRaw = (oItem.actDeliveryDt) ? decodeURIComponent(oItem.actDeliveryDt) : 
							decodeURIComponent(oItem.estDeliveryDt);
						if (item.deliveredRaw == 'Pending') item.deliveredRaw = '';

						var oEvents = oItem.scanEventList;
						var history = [];
						for (var e = 0; e < oEvents.length; e++) {

							var oEvent = oEvents[e];
							if (!oEvent) continue;

							var event = new DeliveryEvent();
							if (oEvent.status) event.status = decodeURIComponent(oEvent.status);
							if (oEvent.scanDetails) {
								if (event.status) event.status += ': ';
								event.status += decodeURIComponent(oEvent.scanDetails);
							}
							event.dateRaw = decodeURIComponent(oEvent.date);
							event.location = decodeURIComponent(oEvent.scanLocation);
							history.push(event);

						}
						item.history = history;
						item.matchNo = decodeURIComponent(oItem.trackingQualifier);
						item.label = decodeURIComponent(oItem.trackingNbr);
						items.push(item);

						if (!isChild && oItem.totalPieces > 1 && oItem.associationInfoList && oItem.associationInfoList[0]) {
							var oAssoc = oItem.associationInfoList[0];
							var url = 'https://www.fedex.com/trackingCal/track';
							var locale = request.options.locale;
							var jsonData = JSON.stringify({
								"AssociatedShipmentRequest":{
									"appType":"WTRK",
									"uniqueKey":"",
									"processingParameters":{
										"anonymousTransaction":true,
										"clientId":"WTRK",
										"returnDetailedErrors":true,
										"returnLocalizedDateTime":false
									},
									"masterTrackingNumberInfo":{
										"trackingNumberInfo":{
											"trackingNumber":decodeURIComponent(oAssoc.trackingNumberInfo.trackingNumber),
											"trackingQualifier":decodeURIComponent(oAssoc.trackingNumberInfo.trackingQualifier),
											"trackingCarrier":decodeURIComponent(oAssoc.trackingNumberInfo.trackingCarrier)
										},
										"associatedType":decodeURIComponent(oAssoc.associatedType)
									}
								}
							});
							var options = new RequestOptions();
							options.data = 'data='+jsonData+'&action=getAssociatedShipments&locale='+locale+'&format=json&version=99';
							this.loadPage(url, this.getFedExMultiple, options);
							break;
						}

					} else {
						if (oItem.errorList && oItem.errorList[0]) errorText = decodeURIComponent(oItem.errorList[0].message);
						success = false;
					}
				}
			} else {
				connect = false;
				success = false;
			}

		} else if (site == 'gls') {

			deliveredPtn = '^DELIVERED';

			if (object && object.tuStatus) {

				var oItem = object.tuStatus[0];
				var item = new DeliveryPackage();

				if (oItem.arrivalTime) {
					item.deliveredRaw = oItem.arrivalTime.value;
				}
				var oProgress = oItem.progressBar;
				if (oProgress) {
					item.status = oProgress.statusText;
					item.deliveredStatus = oProgress.statusInfo;
				}

				var oEvents = oItem.history;
				var history = [];
				if (oEvents) {
					for (var e = 0; e < oEvents.length; e++) {
						var oEvent = oEvents[e];
						var event = new DeliveryEvent();
						event.dateRaw = oEvent.date;
						event.status = oEvent.evtDscr;
						if (oEvent.address) {
							var location = locationFromParts(oEvent.address.city, null, 
								null, oEvent.address.countryName);
							if (location) event.location = location;
						}
						history.push(event);
					}
				}
				item.history = history;
				items.push(item);

			} else if (object.exceptionText) {
				errorText = clearEmptyString(stripHTML(object.exceptionText));
				success = false;
			} else {
				connect = false;
				success = false;
			}

		} else if (site == 'hlg') {

			if (object && object.length > 0) {
				for (var i = 0; i < object.length; i++) {

					var oItem = object[i];
					var item = new DeliveryPackage();
					item.status = oItem.lastStatus.description;
					item.deliveredRaw = oItem.expectedDate;
					if (oItem.statusImageID == 5) {
						item.deliveredStatus = 'Delivered';
					}

					var oEvents = oItem.statusHistory;
					var history = [];
					if (oEvents) {
						for (var e = 0; e < oEvents.length; e++) {
							var oEvent = oEvents[e];
							var event = new DeliveryEvent();
							event.status = oEvent.description;
							event.dateRaw = oEvent.dateTime;
							event.location = locationFromStatus(oEvent.description, from);
							history.push(event);
						}
					}
					item.history = history;
					item.label = oItem.shipmentID;
					items.push(item);

				}
			} else {
				success = false;
			}

		} else if (site == 'la-poste') {

			if (object.code && request.status == 200) {

				var item = new DeliveryPackage();
				// item.url = object.link;
				item.status = object.message;
				if (object.status == 'LIVRE') {
					item.deliveredStatus = 'Delivered';
				}

				var event = new DeliveryEvent();
				event.dateRaw = object.date;
				event.status = object.message;
				item.history = [event];
				items.push(item);

			} else {
				success = false;
				errorText = object.message;				
			}

		} else if (site == 'lasership') {

			deliveredPtn = 'Delivered';

			var oEvents = object.Events;
			if (oEvents && oEvents[0]) {

				var item = new DeliveryPackage();
				item.shippedRaw = object.ReceivedOn;
				item.deliveredRaw = object.EstimatedDeliveryDate;

				var history = [];
				for (var e = 0; e < oEvents.length; e++) {
					var oEvent = oEvents[e];
					var event = new DeliveryEvent();
					event.dateRaw = oEvent.DateTime;
					event.status = oEvent.EventShortText;
					if (oEvent.Location) event.status += ': '+oEvent.Location;
					var location = locationFromParts(oEvent.City, oEvent.State, 
						oEvent.PostalCode, oEvent.Country);
					if (location) event.location = location;
					history.push(event);
				}
				item.history = history;
				items.push(item);

			} else {
				success = false;
				errorText = object.ErrorMessage;				
			}

		} else if (site == 'ontrac') {

			if (object && Array.isArray(object)) {

				for (var i = 0; i < object.length; i++) {

					var oItem = object[i];
					if (oItem.PackageFound == 0) break;

					var item = new DeliveryPackage();
					item.status = oItem.StatuscodeDisplayText;
					item.shippedRaw = oItem.ShippedDate;
					item.deliveredRaw = (oItem.ActualExpectedDeliveryDate) ? oItem.ActualExpectedDeliveryDate : 
						oItem.ExpectedDeliveryDate;

					var trackingNo = oItem.Tracking;
					var runNo = oItem.Run;
					var runLoc = oItem.RunLocation;
					if (trackingNo && runNo) {
						var url = 'https://www.ontrac.com/services/api/TrackingDetails/V1/'+trackingNo+'/'+runNo+'/'+runLoc;
						var options = new RequestOptions('ontrac', i);
						options.service = request.options.service;
						this.loadPage(url, this.getJSON, options);
					}

					item.label = trackingNo;
					items.push(item);

				}
				if (items.length == 0) success = false;

			} else if (isChild && object && object.RunInfo) {

				var item = new DeliveryPackage();
				var oInfo = object.RunInfo;
				item.status = oInfo.StatuscodeDisplayText;
				item.shippedRaw = oInfo.ShippedDate;
				item.deliveredRaw = (oInfo.ActualExpectedDeliveryDate) ? oInfo.ActualExpectedDeliveryDate : 
					oInfo.ExpectedDeliveryDate;

				var oEvents = [];
				if (object.OnTracEvents) oEvents = oEvents.concat(object.OnTracEvents);
				if (object.USPSEvents) oEvents = oEvents.concat(object.USPSEvents);
				var history = [];
				for (var e = 0; e < oEvents.length; e++) {

					var oEvent = oEvents[e];
					if (!oEvent) continue;

					var event = new DeliveryEvent();
					event.status = oEvent.StatuscodeDescriptionText;
					event.dateRaw = oEvent.EventDate;
					event.location = oEvent.EventLocation;
					history.push(event);

				}
				item.history = history;
				items.push(item);

			} else {
				connect = false;
				success = false;
			}

		} else if (site == 'poste-it') {

			var itemStatus = object.sintesiStato;
			if (itemStatus) {

				var item = new DeliveryPackage();
				item.status = itemStatus;
				if (object.stato == '5') {
					item.deliveredStatus = 'Delivered';
				}

				var oEvents = object.listaMovimenti;
				var history = [];
				if (oEvents) {
					for (var e = 0; e < oEvents.length; e++) {
						var oEvent = oEvents[e];
						var event = new DeliveryEvent();
						event.status = oEvent.statoLavorazione;
						var timeStamp = oEvent.dataOra;
						if (timeStamp) event.dateRaw = timeStampToISOString(timeStamp);
						event.location = oEvent.luogo;
						history.unshift(event);
					}
				}
				item.history = history;
				items.push(item);

			} else {
				success = false;
			}

		} else if (site == 'posten-no') {

			if (object && object.consignmentSet) {
				var oItems = object.consignmentSet;
				for (var i = 0; i < oItems.length; i++) {

					var oItem = oItems[i];
					var oSubItems = oItem.packageSet;

					for (var s = 0; s < oSubItems.length; s++) {

						var oSubItem = oSubItems[i];

						var item = new DeliveryPackage();
						item.status = oSubItem.statusDescription;
						item.deliveredRaw = oSubItem.dateOfDelivery || oSubItem.dateOfEstimatedDelivery;

						var oEvents = oSubItem.eventSet;
						var history = [];
						if (oEvents) {
							for (var e = 0; e < oEvents.length; e++) {
								var oEvent = oEvents[e];
								var event = new DeliveryEvent();
								if (e == 0) item.deliveredStatus = oEvent.status;
								event.status = oEvent.description;
								event.dateRaw = oEvent.dateIso;
								var location = locationFromParts(oEvent.city, null, 
									oEvent.postalCode, oEvent.country);
								if (location) event.location = location;
								history.push(event);
							}
						}
						item.history = history;
						item.label = oSubItem.itemId;
						items.push(item);

					}

				}
			} else {
				success = false;
				if (object) errorText = object.errorMessage;
			}

		} else if (site == 'postnl') {

			var itemStatus = object.shipmentPhase;
			if (itemStatus) {

				var item = new DeliveryPackage();
				item.status = itemStatus;
				item.shippedRaw = object.sentDate;
				item.deliveredRaw = object.deliveredDate;
				if (!item.deliveredRaw && object.deliveryExpectation) {
					item.deliveredRaw = object.deliveryExpectation.deliveryDateUntil;
				}

				var oEvents = object.trackingEvents;
				var history = [];
				if (oEvents) {
					for (var e = 0; e < oEvents.length; e++) {
						var oEvent = oEvents[e];
						var event = new DeliveryEvent();
						event.status = oEvent.description;
						event.dateRaw = oEvent.dateTime;
						var location = oEvent.location;
						if (location) event.status += ': '+location;
						event.location = location;
						history.push(event);
					}
				}
				item.history = history;
				items.push(item);

			} else {
				success = false;
			}

		} else if (site == 'postnord') {

			if (object) object = object.TrackingInformationResponse;
			if (object && object.shipments && object.shipments.length > 0) {
				var oItems = object.shipments;
				for (var i = 0; i < oItems.length; i++) {

					var oItem = oItems[i];
					var oSubItems = oItem.items;

					for (var s = 0; s < oSubItems.length; s++) {

						var oSubItem = oSubItems[s];
						var item = new DeliveryPackage();
						item.status = oItem.statusText.body;
						item.deliveredRaw = oItem.deliveryDate || oItem.estimatedTimeOfArrival;
						if (oItem.status == 'DELIVERED') {
							item.deliveredStatus = 'Delivered';
						}

						var oEvents = oSubItem.events;
						var history = [];
						if (oEvents) {
							for (var e = 0; e < oEvents.length; e++) {
								var oEvent = oEvents[e];
								var event = new DeliveryEvent();
								event.status = oEvent.eventDescription;
								event.dateRaw = oEvent.eventTime;
								var oLoc = oEvent.location;
								if (oLoc) {
									var city = oLoc.city;
									if (!city && oLoc.displayName != oLoc.country) city = oLoc.displayName;
									var location = locationFromParts(city, null, oLoc.postcode, oLoc.country);
									event.location = location;
									if (oEvent.status == 'INFORMED') event.ignoreLocation = true;
								}
								history.unshift(event);
							}
						}
						item.history = history;
						item.label = oSubItem.itemId;
						items.push(item);

					}

				}
			} else {
				success = false;
			}

		} else if (site == 'tnt') {

			if (object) object = object['tracker.output'];
			if (object && object.consignment) {
				var oItems = object.consignment;
				for (var i = 0; i < oItems.length; i++) {

					var oItem = oItems[i];
					var item = new DeliveryPackage();
					item.shippedRaw = object.originDate;
					item.deliveredRaw = oItem.destinationDate;
					if (oItem.status && oItem.status.isDelivered) {
						item.deliveredStatus = 'Delivered';
					}

					var oEvents = oItem.events;
					var history = [];
					if (oEvents) {
						for (var e = 0; e < oEvents.length; e++) {
							var oEvent = oEvents[e];
							var event = new DeliveryEvent();
							event.status = oEvent.statusDescription;
							event.dateRaw = oEvent.date;
							var oLoc = oEvent.location;
							if (oLoc) {
								var location = locationFromParts(oLoc.city, null, null, oLoc.country);
								event.location = location;
							}
							history.push(event);
						}
					}
					item.history = history;
					item.label = oItem.consignmentNumber;
					items.push(item);

				}
			} else {
				success = false;
			}

		} else if (site == 'ups') {

			parseOptions.locale = request.options.locale;

			if (object && object.trackDetails) {
				var oItems = object.trackDetails;
				for (var i = 0; i < oItems.length; i++) {

					var oItem = object.trackDetails[i];
					if (oItem.errorCode) {
						success = false;
						errorText = oItem.errorText;
						break;
					}

					var item = new DeliveryPackage();
					var oAdditional = oItem.additionalInformation;
					item.status = oItem.packageStatus;
					item.shippedRaw = (oAdditional) ? oAdditional.shippedOrBilledDate : null;
					item.deliveredRaw = oItem.deliveredDate || oItem.scheduledDeliveryDate;
					item.deliveredStatus = oItem.progressBarType;

					var oEvents = oItem.shipmentProgressActivities;
					var history = [];
					if (oEvents) {

						for (var e = 0; e < oEvents.length; e++) {
							var oEvent = oEvents[e];
							if (oEvent.activityScan == null) continue;
							var event = new DeliveryEvent();
							var status = trim(oEvent.activityScan);
							var extra = oEvent.activityAdditionalDescription;
							var milestoneName = (oEvent.milestone) ? oEvent.milestone.name : null;
							if (milestoneName == 'cms.stapp.delivered') {
								if (item.status && status == item.status.toUpperCase()) status = item.status;
								if (!extra) extra = oItem.leftAt;
							}
							if (extra) status += ': '+extra;
							event.status = status;
							event.dateRaw = oEvent.date;
							event.location = oEvent.location;
							history.push(event);
						}

					} else if (oItem.trackSummaryView) {

						// InfoNotice
						var oSummary = oItem.trackSummaryView;
						item.status = oSummary.packageStatus;
						item.deliveredStatus = oSummary.packageStatusIconType;
						item.ignoreHistory = true;
						var event = new DeliveryEvent();
						var status = oSummary.packageStatus;
						var extra = oSummary.leftAt;
						if (extra) status += ': '+extra;
						event.status = status;
						event.dateRaw = oSummary.packageStatusDateWithYear;
						history.push(event);

					}

					item.history = history;
					item.label = oItem.trackingNumber;
					items.push(item);

				}
			} else {
				success = false;
				if (object) errorText = object.statusText;
			}

		}

		if (!isChild) delivery.items = [];
		var itemCount = (isChild) ? 1 : items.length;
		for (var i = 0; i < itemCount; i++) {

			var itemNo = (isChild) ? childNo : i;
			var item = items[i];
			if (!item) break;

			var details = null;
			if (item.history && item.history.length > 0) {
				var route = [];
				for (var e = 0; e < item.history.length; e++) {
					var event = item.history[e];
					if (event.dateRaw) {
						var date = parseDate(event.dateRaw, parseOptions);
						event.dateString = (date) ? date.toISOString() : null;
					}
					if (event.location && !event.ignoreLocation) {
						var location = parseLocation(event.location, parseOptions);
						route.push(location);
					}
				}
				details = item.history[0];
				details.route = route;
				if (item.ignoreHistory) item.history = null;
			} else {
				details = {};
			}

			if (!item.status) item.status = details.status;
			if (!item.deliveredStatus) item.deliveredStatus = item.status;
			if (item.deliveredStatus && item.deliveredStatus.match(new RegExp(deliveredPtn, 'i'))) item.status = 'Complete';
			var isComplete = (item.status == 'Complete');
			if (isComplete && !item.deliveredRaw) item.deliveredRaw = details.dateRaw;

			if (isComplete && item.deliveredLocation) details.location = item.deliveredLocation;
			if (details.location == undefined) details.location = '';
			details.mapLocation = (details.location && !details.ignoreLocation) ? parseLocation(details.location, parseOptions) : '';
			item.details = details;

			if (success) {
				var dItem = (isChild) ? delivery.items[itemNo] : new DeliveryPackage();
				if (!isChild && itemCount > 1) {
					dItem.label = item.label;
				}
				if (item.status) dItem.status = item.status;
				if (item.shippedRaw) dItem.shippedDate = parseDate(item.shippedRaw, parseOptions);
				if (item.deliveredRaw) dItem.deliveredDate = parseDate(item.deliveredRaw, parseOptions, !isComplete);
				if (item.details) dItem.details = item.details;
				if (item.history) dItem.history = item.history;
				if (item.matchNo) dItem.matchNo = item.matchNo;
				delivery.items[itemNo] = dItem;
			}

		}

		if (isChild) {
			if (!connect) {
				this.updateFailed(DeliveryError.connect, null);
			}
		} else {
			if (!success) {
				var error = DeliveryError.notFound;
				logRequest(1,request);
				delivery.tries = 0;
				if (connect) {
					delivery.count = 0;
				} else if (delivery.count) {
					error = DeliveryError.connect;
				} else {
					error = DeliveryError.server;
				}
				this.updateFailed(error, errorText);
			} else {
				delivery.count = itemCount;
				delivery.updatePackageNames();
			}
		}

	} else if (isChild) {

		logRequest(1,request);
		this.updateFailed(DeliveryError.notFound, null);

	} else {

		var error = DeliveryError.notFound;
		logRequest(1,request);
		if (!delivery.count) {
			if (site == 'postnl' && (request.status == 400 || request.status == 404)) {
				delivery.count = 0;
			} else if (site == 'hlg' && request.status == 204) {
				delivery.count = 0;
			} else {
				error = DeliveryError.server;
			}
		}
		this.updateFailed(error, null);

	}

	this.reqComplete++;
	this.updateItems();

}

Delivery.prototype.getHTMLJSON = function (request) {

	var delivery = this;
	var options = request.options;
	var site = options.site;

	var output = trim(request.responseText);

	if ((request.status == 200 || request.status == 0) && output) {

		var html = '';
		var object = {};

		try {
			object = JSON.parse(output);
		} catch (err) {
			logMessage(1,err);
			logRequest(1,request);
		}

		if (site == 'chronopost') {
			if (object.top) html += object.top + ' '
			if (object.tab) html += object.tab
		}

		this.loadSection(html, options);

	} else {
		logRequest(1,request);
		this.updateFailed(DeliveryError.server, null);
	}

	this.reqComplete++;
	this.updateItems();

}

Delivery.prototype.getMultiple = function (request) {

	var delivery = this;
	var sections = false;
	var site = request.options.site;
	var commonPtn;
	var commonOutput;
	var packagePtn;
	var packageOverlapPtn;
	var urlPtn;
	var urlReplace;
	var baseURL;
	var labelPtn;

	if ((request.status == 200 || request.status == 0) && request.responseText) {

		var output = stripWhiteSpace(request.responseText);

		if (site == 'parcelforce') {

			packagePtn = '<tr[^>]*> ?<td> ?<a href="[^"]*"[^>]*>[^<]*</a> ?</td> ?<td>[^<]*</td> ?<td>[^<]*</td> ?<td>[^<]*</td> ?<td>[^<]*</td> ?</tr>';
			urlPtn = '<a href="([^"]*)"[^>]*>';
			labelPtn = '<a href="[^"]*"[^>]*>([^<]*)</a>';
			baseURL = 'https://www.parcelforce.com';

		} else if (site == 'post-at') {

			packagePtn = '<tr> ?<td[^>]*> ?<a href="[^"]+">[0-9]+</a> ?</td> ?<td[^>]*>[^<]*</td> ?<td[^>]*>[^<]*</td> ?<td[^>]*>[^<]*</td> ?</tr>';
			urlPtn = '<a href="([^"]+)">';
			labelPtn = '<a href="[^"]+">([0-9]+)</a>';	
			baseURL = 'https://www.post.at';		

		} else if (site == 'purolator') {

			packagePtn = "var jsSummaryTable = \\[ \\['([A-Z0-9]*)','([0-9-]+)','([A-Z0-9]+)'";
			urlPtn = "\\['([A-Z0-9]*)','([0-9-]+)','([A-Z0-9]+)'";
			urlReplace = 'https://www.purolator.com/'+request.options.lang+'/ship-track/tracking-details.page?pin=$3&sdate=$2';

		} else if (site == 'yamato') {

			commonPtn = '<table class="saisin">(.*?)</table>(.*?)<table class="meisaibase">(.*?)</table>';
			packageOverlapPtn = '<tr> ?<td[^>]* rowspan="2"> ?([0-9]+) ?<\/td>(.*?)(<tr> ?<td[^>]* rowspan="2">|<\/table>)';
			labelPtn = '<tr> ?<td[^>]* rowspan="2"> ?([0-9]+)';

		}

		if (commonPtn) {
			var result = output.match(new RegExp(commonPtn, 'i'));
			if (result) commonOutput = result[0];
		}

		var packages;
		if (packagePtn) {
			packages = output.match(new RegExp(packagePtn, 'gi'));
		} else if (packageOverlapPtn) {
			packages = overlappingMatches(output, packageOverlapPtn);
		}

		if (packages) {
			for (var y in packages) {

				var packageOutput = packages[y];
				var item = new DeliveryPackage();
				if (y == 0) delivery.items = [];

				var label = false;
				if (labelPtn) {
					var regEx = new RegExp(labelPtn, 'i');
					var result = packageOutput.match(regEx);
					if (result) label = trim(result[1]);
				}
				item.label = label;
				delivery.items[y] = item;

				var options = new RequestOptions(site, y);
				var urlRegEx = null;
				if (urlPtn) urlRegEx = new RegExp(urlPtn, 'i');

				if (urlRegEx && (result = packageOutput.match(urlRegEx))) {
					var url;
					if (urlReplace) {
						url = result[0].replace(urlRegEx, urlReplace);
					} else {
						url = result[1];
					}
					url = url.replace(/&amp;/g,'&');
					if (baseURL && !url.match(/^https?:\/\//i)) {
						if (url.charAt(0) != '/') url = '/'+url;
						url = baseURL+url;
					}
					this.loadPage(url, this.getTracker, options);
				} else {
					if (commonOutput) packageOutput = commonOutput+' '+packageOutput;
					this.loadSection(packageOutput, options);
				}

			}
			delivery.count = delivery.items.length;
			delivery.updatePackageNames();
		} else {
			this.reqCount++;
			this.getTracker(request);
		}

	} else {
		this.reqCount++;
		this.getTracker(request);
	}

	this.reqComplete++;
	this.updateItems();

}

Delivery.prototype.getTracker = function (request) {

	var delivery = this;
	var result = false;
	var site = request.options.site;
	var childNo = request.options.childNo;
	var isChild = request.options.isChild;

	if ((request.status == 200 || request.status == 0) && request.responseText) {

		var from = (request.options.service) ? request.options.service.from : delivery.from;
		var options = new ParseOptions(from);

		var statusPtn;
		var ignoreStatusPtn;

		var eventPtn;
		var eventDateNo;
		var eventTimeNo;
		var eventStatusNo;
		var eventStatusNoExtra = -1;
		var eventLocNo;
		var eventLocNoExtra = -1;
		var eventLocPtn;
		var latestFirst = true;
		var fillEventDates = false;

		var shippedDatePtn;
		var deliveredDatePtn; // Preferred delivery date
		var deliveredDateAltPtn; // Used if the main date is not found
		var useEventDateWhenDelivered = false;
		var splitDateRanges = false;
		var deliverLocPtn;
		var deliveredStatusPtn;
		var deliveredPtn = 'Delivered';

		var errorPtn;
		var connectPtn;
		var validCacheURLPtn;

		var foundStatus = false;
		var status;
		var details;
		var history;
		var shippedRaw;
		var deliveredRaw;
		var errorText;

		var output = stripWhiteSpace(request.responseText);

		if (site == 'amazon-track') {

			output = output.replace(/<span class="nowrap"><\/span>/gi,'');
			options.from = getLanguageFrom(from,output);
			statusPtn = '<span id="primaryStatus"[^>]*>(.*?)(?:</span> ?<span id="secondaryStatus">|</div>)';
			if (output.match(new RegExp(statusPtn, 'i'))) {
				eventPtn = '(?:<span class="tracking-event-date">([^<]*)</span> ?</div> ?)?<div[^>]*> ?<div[^>]*> ?<span class="tracking-event-time">([^<]*)</span> ?<span[^>]*></span> ?</div> ?<div[^>]*> ?<div[^>]*> ?<span class="tracking-event-message">([^<]*)</span> ?</div> ?<div[^>]*> ?<span class="tracking-event-location">([^<]*)</span> ?</div>';
				eventDateNo = 1;
				eventLocNo = 4;
				eventStatusNo = 3;
				// shippedDatePtn = '<span class="milestone-primaryMessage[^"]*"> ?(?:Expected to ship|Shipped) (?:<span [^>]*>)?([^<]*)';
				deliveredDatePtn = '(?:<span id="primaryStatus"[^>]*>|<span class="milestone-primaryMessage[^"]*">) ?(?:Arriving|Expected(?: by)?|Now expected|Zustellung|Erwartete Zustellung|Voraussichtlich|Voraussichtliche Lieferung|Entrega(?: el)?|Entrega prevista|Llegada(?: el| antes del)?|Livraison(?: prévue| estimée)?(?: le)?|En cours d\'acheminement|Arriverà|In consegna|In arrivo(?: il)?|Delivered|Zugestellt|Entregado|Livré|Consegnato|配達を完了しました|配達完了):? ?(.*?)(?:</span> ?<span id="secondaryStatus">|</div>)';
				deliveredDateAltPtn = '(?:<span id="primaryStatus"[^>]*>|<span class="milestone-primaryMessage[^"]*">) ? ?([0-9]+月[0-9]+日|[0-9/]*)[^<]*に到着予定';
				deliveredStatusPtn = '"lastReachedMilestone" ?: ?"([^"]*)"';
				deliveredPtn = '^DELIVERED$';
				ignoreStatusPtn = '^ ?(?:Arriving|Expected(?: by)?|Now expected|Zustellung|Erwartete Zustellung|Voraussichtlich|Voraussichtliche Lieferung|Entrega(?: el)?|Entrega prevista|Llegada(?: el| antes del)?|Livraison(?: prévue| estimée)?(?: le)?|Arriverà|In consegna|Delivered|Zugestellt|Entregado|Livré|Consegnato|配達を完了しました|配達完了)';
			} else {
				statusPtn = '<div id="summaryLeft"[^>]*> ?<h2[^>]*>([^<]*)';
				eventPtn = '<div id="summaryRight"[^>]*> <span[^>]*>([^<]*)<span class="a-size-mini a-color-secondary">([^<]*)</span>';
				eventDateNo = 2;
				eventStatusNo = 1;
				// Expected delivery:|Delivered on:
				deliveredDatePtn = '<span class="a-color-success a-text-bold">([^<]*)</span>';
				deliveredStatusPtn = 'currentBeadType ?: ?"([^<"]*)"';
				deliveredPtn = '^COMPLETED$';
			}
			splitDateRanges = true;
			useEventDateWhenDelivered = true;
			fillEventDates = true;
			validCacheURLPtn = '(packageIndex|orderingShipmentId)=[0-9]+';

		} else if (site == 'apple-inline') {

			eventPtn = '<td[^>]*class=\\\\"tdata current-status status-notification\\\\">([^<]*)<\\\\/td> ?(<td[^>]*> ?<\\\\/td> ?)?<td class=\\\\"tdata current-status date\\\\">([^<]*)<\\\\/td>';
			eventDateNo = 3;
			eventStatusNo = 1;
			deliveredDatePtn = '<th scope=\\\\"row\\\\" class=\\\\"text-alt tdata\\\\">(?:Est. Delivery|配送の目安)<\\\\/th> ?<td colspan=\\\\"3\\\\" class=\\\\"tdata current-status date\\\\">([^<]*)<\\\\/td>';
			deliveredPtn = '^(Delivered|配送済み)$';
			useEventDateWhenDelivered = true;

		} else if (site == 'chronopost') {

			options.lang = request.options.lang;
			statusPtn = '<div class="ch-colis-information"> ?<img [^>]*>([^<]*)';
			eventPtn = '<tr class="toggleElmt"> ?<td>([^<]*)<br[^>]*>([^<]*)</td> ?<td>([^<]*)<br[^>]*>([^<]*)</td> ?<td[^>]*>';
			eventDateNo = 1;
			eventTimeNo = 2;
			eventLocNo = 3;
			eventStatusNo = 4;
			deliveredPtn = 'Delivered|Livraison effectuée|Livraison effectu&eacute;e';
			errorPtn = '<div class="ch-colis-information"> ?<img [^>]*>([^<]*)';

		} else if (site == 'gls') {

			eventPtn = '<tr class="resulthead">.*?<tr[^>]*> ?<td>([^<]*)</td> ?<td> ?(?:[a-z]{2} [0-9]{3} )?(?:, )?([^<]*)</td> ?<td>[^<]*</td> ?<td>([^<]*)';
			eventDateNo = 1;
			eventLocNo = 2;
			eventStatusNo = 3;
			shippedDatePtn = '(?:Pick-Up Date|Abhol-Datum|Date enlèvement|Afhentningsdato): ?</td> ?<td class="value"> ?([^<]*)';
			deliveredDatePtn = '(?:Delivery Date|Zustell-Datum|Date livraison|Leveringsdato): ?</td> ?<td class="value"> ?([^<]*)';
			deliveredPtn = '^(Delivered|Zugestellt|LIVRE|Forsendelsen er leveret)';
			connectPtn = '<h1> ?Temporarily out of order ?</h1>';

		} else if (site == 'japan-post') {

			eventPtn = '<tr> ?<td[^>]*>([^<]*)</td> ?<td[^>]*>([^<]*)</td> ?<td[^>]*>([^<]*)</td> ?<td[^>]*>([^<]*)</td> ?<td[^>]*>([^<]*)</td> ?</tr> ?<tr> ?<td[^>]*>([^<]*)</td> ?</tr>';
			eventDateNo = 1;
			eventLocNo = 4;
			eventLocNoExtra = 5;
			eventStatusNo = 2;
			eventStatusNoExtra = 3;
			deliveredDatePtn = '(?:Scheduled delivery date:|配達予定日：) ?([^<]*)';
			deliveredPtn = '(Final delivery|お届け済み|お届け先にお届け済み|窓口でお渡し)';
			errorPtn = '<td colspan="5">([^<]*)</td>';
			latestFirst = false;

		} else if (site == 'parcelforce') {

			eventPtn = '<tr[^>]*> ?<td[^>]*>([^<]*)</td> ?<td[^>]*>([^<]*)</td> ?<td[^>]*>([^<]*)</td> ?<td[^>]*>(.*?)</td> ?</tr>';
			if (output.match(new RegExp(eventPtn, 'i'))) {
				statusPtn = '<dt[^>]*> ?Tracking status: ?</dt> ?<dd>([^<]*)</dd>';
				eventDateNo = 1;
				eventTimeNo = 2;
				eventLocNo = 3;
				eventStatusNo = 4;
				deliveredDatePtn = '<dt[^>]*> ?Delivered on: ?</dt><dd>([^<]*)</dd>';
			} else {
				statusPtn = '<h3 class="track-trace-rml-messages">([^<]*)';
				eventPtn = '<h3 class="track-trace-rml-messages">[^<]*</h3> ?<p>([^<]*)';
				eventStatusNo = 1;
				deliveredDatePtn = 'was delivered [a-z\\s]* on ([0-9/]*)';
			}
			errorPtn = '<h2> ?Please review the information below ?</h2> ?<ul> ?<li>([^<]*)';
			connectPtn = '<p> ?(Our website is temporarily unavailable.|Because of very heavy site traffic, this page is currently unavailable.)';

		} else if (site == 'post-at') {

			statusPtn = '<h4 class="sendungsstatus-header">(.*?)</h4>';
			if (output.match(new RegExp(statusPtn, 'i'))) {
				eventPtn = '<div class="media-body"> ?<em> ?<b> ?[A-Z:]* ?([0-9.]+)&nbsp;([0-9:]*) ?</b> ?</em> ?<br> ?<b>([^<]*)</b>';
				eventDateNo = 1;
				eventTimeNo = 2;
				eventStatusNo = 3;
				eventLocPtn = '; ?([A-Z]* ?[0-9]+(; ?[A-Z]*)?) ?$';
				deliveredDatePtn = '<div[^>]*> ?(?:Voraussichtliche Zustellung|Voraussichtliche&nbsp;&shy;Zustellung|Expected delivery day): ?</div> ?<div[^>]*> ?(?:zugestellt am|delivered on)? ?[a-z\\s]*, ([0-9./]*)';
				deliveredStatusPtn = '<li class="sendungsstatus-progress-item ([^"]*)"> ?<span[^>]*> ?</span> ?<span class="pi pi-icon_005[^"]*"[^>]*>';
				deliveredPtn = '^(active|complete)$';
			} else {
				eventPtn = '(?:Sendungsstatus|Delivery status): ?([^<;]*)<br> ?(?:Datum|Date): ?([^<;]*) ?; ?(?:PLZ|Postcode): ?([^<;]*)';
				eventDateNo = 2;
				eventStatusNo = 1;
				eventLocNo = 3;
				deliveredPtn = '^(übergeben|delivered)$';
			}
			connectPtn = '<h4> ?Die Seite kann nicht angezeigt werden|<h3> ?DB Error'; // This has probably changed

		} else if (site == 'purolator') {

			statusPtn = "\"status\" ?: ?'([^']*)'";
			eventPtn = "\\[ ?'([^']*)', ?'([^']*)', ?'([^']*)', ?'([^']*)' ?\\]";
			eventDateNo = 1;
			eventLocNo = 3;
			eventStatusNo = 4;
			deliveredDatePtn = '"deliverBy" ?: ?"([^"]*)"';
			deliveredPtn = '^Delivered';
			connectPtn = 'addFeedbackError\\("#feedbackTrackDetails","error\\.service\\.1031990052"\\)';

		} else if (site == 'sagawa') {

			output = decodeDecimalEntities(output);

			eventPtn = '<tr> ?<td> ?[⇒↓]([^<]*)</td> ?<td>([^<]*)</td> ?<td>([^<]*)</td> ?</tr>';
			eventDateNo = 2;
			eventStatusNo = 1;
			eventLocNo = 3;
			deliveredDatePtn = '<span[^>]*>[^<]*</span> ?</th> ?<th[^>]*> ?(?:再配達希望日時) ?</th> ?</tr> ?<tr> ?<td[^>]*>[^<]*(?:<div[^>]*> ?― ?</div>)?[^<]*</td> ?<td[^>]*>([^<]*)</td>';
			deliveredDateAltPtn = '<span[^>]*> ?(?:配達完了日|お届け予定) ?</span> ?</th> ?<th[^>]*>[^<]*</th> ?</tr> ?<tr> ?<td[^>]*>([^<]*)</td>';
			deliveredPtn = '(配達完了)'; // 配達は終了致しました。|配達終了
			errorPtn = '<tr> ?<th>詳細表示</th> ?<td>([^<]*)</td> ?</tr>';
			latestFirst = false;

		} else if (site == 'swiss-post') {

			eventPtn = '<tr[^>]*> ?<td[^>]*>(<span[^>]*>[^<]*</span>)?([^<]*)</td> ?<td[^>]*>([^<]*)</td> ?<td class="event_event"> ?(<span[^>]*>[^<]*</span>)?(.*?) ?(<img[^>]*> ?)?</td> ?<td[^>]*>([^<]*)</td> ?<td[^>]*>([^<]*)</td> ?</tr>';
			eventDateNo = 2;
			eventLocNo = 7;
			eventLocNoExtra = 8;
			eventStatusNo = 5;
			deliveredDatePtn = '<dt>(?:Predicted delivery date|Voraussichtlicher Zustelltag|Jour de distribution prévu|Giorno di recapito previsto):</dt> ?<dd>([^<]*)</dd>';
			deliveredPtn = '^(Delivered|Zugestellt|Distribué|Distribuito|Recapitato)';
			latestFirst = false;

		} else if (site == 'tnt-express') {

			eventPtn = '<tr[^>]*> ?<td>([^<]*)</td> ?<td>([^<]*)</td> ?<td>([^<]*)</td> ?<td>([^<]*)</td>';
			eventDateNo = 2;
			// eventTimeNo = 3; // Sometimes times are missing so don't use them for sorting
			eventLocNo = 4;
			eventStatusNo = 1;
			deliveredDatePtn = "<td class='strong'>Estimated Delivery by:([^<]*)";
			deliveredPtn = "(^Delivered|^Dlvrd|^We've delivered your shipment|^Proof of Delivery Image$|Agent Collected \\(Full\\)$)";
			latestFirst = false;

		} else if (site == 'usps') {

			eventPtn = '<(?:TrackSummary|TrackDetail)>(<EventTime>([^<]*)</EventTime>|<EventTime/>)(<EventDate>([^<]*)</EventDate>|<EventDate/>)<Event>([^<]*)</Event>((?:<EventCity>|<EventCity/>).*?(?:<EventCountry/>|</EventCountry>))';
			eventDateNo = 4;
			eventLocNo = 6;
			eventStatusNo = 5;
			deliveredDatePtn = '<(?:PredictedDeliveryDate)>([^<]*)';
			deliveredDateAltPtn = '<(?:DeliveryNotificationDate|ExpectedDeliveryDate|GuaranteedDeliveryDate)>([^<]*)';
			deliveredStatusPtn = '<StatusCategory>([^<]*)';
			if (output.match(new RegExp(deliveredStatusPtn, 'i'))) {
				deliveredPtn = '^(Delivered|Delivered to Agent)$';
			} else {
				// For older clients
				deliveredStatusPtn = null;
				deliveredPtn = '^Delivered';
			}
			errorPtn = '<Error><Number>[^<]*</Number>(?:<Source>[^<]*</Source>)?<Description>([^<]*)</Description>';
			connectPtn = '<Error><Number>-2147220704</Number>';

		} else if (site == 'usps-legacy') {

			eventPtn = '<span> ?<strong>([^<]*)</strong> ?</span> ?<br /> ?(?:<span>)?([^<]*)</span> ?<br /> ?(?:(?:<span>)?([^<]*)</span> ?<br[^>]*>)? ?(?:<span>([^<]*)</span> ?<br[^>]*>)? ?<hr[^>]*>';
			eventDateNo = 1;
			eventLocNo = 3;
			eventStatusNo = 2;
			deliveredDatePtn = '<span class="eta_wrap"> ?<span[^>]*> <em[^>]*>[^<]*</em> (<strong[^>]*>[^<]*</strong> <span[^>]*> <span>[^<]*</span> [^<]*)';
			deliveredStatusPtn = '<div class="product_summary delivery_([a-z]*)">';
			errorPtn = '<h2> <strong>[^<]*</strong> <p[^>]*>([^<]*)';
			connectPtn = "(To learn about integrating the free Postal Service&reg; Address and Tracking API's into your application, please visit|This service is currently unavailable|USPS.com is temporarily unavailable.|Sorry, the tracking system is having technical difficulties. Please try your search again later.)";

		} else if (site == 'ups-mi') {

			eventPtn = '<tr> ?<td class="LeftD_BottomL"[^>]*>(.*?)</td> ?<td class="LeftL_Bottom[^"]*"[^>]*>(.*?)</td> ?<td class="LeftL_Bottom[^"]*"[^>]*>(.*?)</td> ?</tr>';
			eventDateNo = 1;
			eventLocNo = 3;
			eventStatusNo = 2;
			deliveredDatePtn = '<span id="[a-z0-9_]*ProjDelDate">([^<]*)';

		} else if (site == 'yamato') {

			statusPtn = '<table class="saisin">.*?<td class="font14"> ?(?!伝票番号未登録)([^<]*)';
			eventPtn = '<tr[^>]*> ?(<td[^>]* rowspan="2"> ?[0-9]+ ?</td>)? ?<td[^>]*> ?<(img|image)[^>]*> ?</td> ?<td[^>]*>([^<]*)(<br>)?</td> ?<td[^>]*>([^<]*)(<br>)?</td> ?<td[^>]*>([^<]*)(<br>)?</td> ?<td[^>]*>(<a[^>]*>)?([^<]*)(</a>)?(<br>)?</td> ?<td[^>]*>([^<]*)(<br>)?</td> ?(<td[^>]*>(<br>)?</td>)? ?(<td[^>]* rowspan="2">[^<]*</td>)? ?</tr>';
			eventDateNo = 5;
			eventLocNo = 10;
			eventStatusNo = 3;
			deliveredDatePtn = '(?:お届け予定日時 ?</th> ?(?:<th[^>]*> ?お届け希望日時 ?</th> ?)?</tr> ?<tr> ?<td[^>]*>[^<]*<br> ?</td> ?<td[^>]*>|<tr> ?<td>[^<]*</td> ?<td>[^<]*(?:<br>)?</td> ?<td>[^<]*(?:<br>)?</td> ?<td>(?:<a[^>]*>)?[^<]*(?:</a>)?</td> ?<td>[^<]*</td> ?<td>)([^<]*)';
			deliveredPtn = '^(配達完了|投函完了)';
			errorPtn = '<table class="saisin">.*?<td class="font14">([^<]*)';
			latestFirst = false;
			useEventDateWhenDelivered = true;

		} else if (site == 'yodel') {

			statusPtn = '<div class="parcel-current-status-message">([^<]*)';
			eventPtn = '<div class="row[^"]*"> ?<div[^>]*>[^<]*</div> ?</div> ?<div[^>]*> ?<div[^>]*> ?<div[^>]*> ?<div class="tracking-event-date-year">([^<]*)</div> ?<div class="tracking-event-date-time">([^<]*)</div> ?</div> ?</div> ?<div[^>]*> ?<div class="tracking-event-description">([^<]*)</div> ?</div> ?</div>';
			eventDateNo = 1;
			eventTimeNo = 2;
			eventStatusNo = 3;
			deliveredDatePtn = '<div class="estimated-delivery-date-value parcel-status-value">([^<]*)';
			deliveredStatusPtn = '<div id="box-4"[^>]*> ?<i class="yodelicon yodelicon-([a-z]+)-stage"> ?</i>';
			deliveredPtn = '^ok$';
			errorPtn = '<div class="error-msg error-msg--general">([^<]*)';
			useEventDateWhenDelivered = true;

		}

		if (!isChild) {
			var item = new DeliveryPackage();
			delivery.items = [item];
			delivery.updatePackageNames();
		}

		var eventRegEx = null;
		if (eventPtn) {
			eventRegEx = new RegExp(eventPtn, 'i');
		}

		// Get the ship date
		if (shippedDatePtn) {
			var regEx = new RegExp(shippedDatePtn, 'i');
			var result = output.match(regEx);
			if (result && result[1]) shippedRaw = result[1];
		}

		// Get the delivery date
		if (deliveredDatePtn) {
			var regEx = new RegExp(deliveredDatePtn, 'i');
			var result = output.match(regEx);
			if (!result && deliveredDateAltPtn) {
				var regEx = new RegExp(deliveredDateAltPtn, 'i');
				result = output.match(regEx);
			}
			if (result && result[1]) {
				deliveredRaw = stripHTML(result[1]);
				if (deliveredRaw && splitDateRanges) deliveredRaw = endDateInRange(deliveredRaw);
			}
		}

		// Find events for details, history, and route
		if (eventPtn && (result = output.match(new RegExp(eventPtn, 'gi')))) {
			var events = [];
			var lastEvent = null;
			for (var x in result) {
				var event = getTrackerEvent(result[x], options, eventRegEx, eventStatusNo, 
					eventStatusNoExtra, eventDateNo, eventTimeNo, eventLocNo, eventLocNoExtra, 
					eventLocPtn);
				if (event) {
					var date = parseDate(event.dateRaw, options, false);
					event.dateString = (date) ? date.toISOString() : null;
					event.index = x;
					if (fillEventDates) {
						if (!date && lastEvent) {
							event.dateString = lastEvent.dateString;
						}
						lastEvent = event;
					}
					events.push(event);
				}
			}
			if (!latestFirst) {
				events.sort(function(a,b){
					return (latestFirst) ? a.index - b.index : b.index - a.index;
				});
			}
			var route = [];
			for (var x = 0; x < events.length; x++) {
				var location = events[x].location;
				if (location) route.push(parseLocation(location, options));
			}
			details = events[0];
			details.route = route;
			history = events;
		}

		// Get the current status if it's separate, or use the detail status
		if (statusPtn) {
			var statusRegEx = new RegExp(statusPtn, 'i');
			if (result = output.match(statusRegEx)) {
				if (result[1]) {
					foundStatus = true;
					status = result[1];
				}
				status = clearEmptyString(parseStatus(status, options));
				if (ignoreStatusPtn && status.match(new RegExp(ignoreStatusPtn, 'i'))) status = null;
			}
		} else if (details) {
			status = details.status;
		}

		if (!status && details) status = details.status;
		if (deliveredStatusPtn) {
			var regEx = new RegExp(deliveredStatusPtn, 'i');
			var result = output.match(regEx);
			if (result && result[1] && result[1].match(new RegExp(deliveredPtn, 'i'))) status = 'Complete';
		} else if (status && status.match(new RegExp(deliveredPtn, 'i'))) {
			status = 'Complete';
		}
		var isComplete = (status == 'Complete');

		var shippedDate = parseDate(shippedRaw, options);
		var deliveredDate = parseDate(deliveredRaw, options, !isComplete);

		if (details) {
			if (isComplete) {
				if (!deliveredDate || useEventDateWhenDelivered) {
					deliveredDate = dateFromISOString(details.dateString);
				}
				if (deliverLocPtn) {
					var regEx = new RegExp(deliverLocPtn, 'i');
					result = output.match(regEx);
					if (result && result[1]) details.location = result[1];
				}
			}
			details.mapLocation = (details.location) ? parseLocation(details.location, options) : '';
		}

		if (delivery.items[childNo]) {
			if (status) delivery.items[childNo].status = status;
			if (details) delivery.items[childNo].details = details;
			if (history) delivery.items[childNo].history = history;
			if (shippedDate) delivery.items[childNo].shippedDate = shippedDate;
			if (deliveredDate) delivery.items[childNo].deliveredDate = deliveredDate;
			if (delivery.useItemCache) delivery.items[childNo].url = request.url;
		}

		if (isChild) {
			if (delivery.usedItemCache && !foundStatus && !shippedRaw && !deliveredRaw && !details) {
				logRequest(1,request);
				logMessage(1,'Cached URL failed to load: '+request.url);
				delivery.clearItemCache = true;
			} else if (delivery.usedItemCache && details && details.status && validCacheURLPtn && !request.url.match(new RegExp(validCacheURLPtn))) {
				logMessage(1,'Cached URL needs to be updated: '+request.url);
				delivery.clearItemCache = true;
			} else if (connectPtn && output.match(new RegExp(connectPtn, 'i'))) {
				this.updateFailed(DeliveryError.notFound, null);
			}
		} else {
			if (!foundStatus && !shippedRaw && !deliveredRaw && !details) {
				logRequest(1,request);
				if (connectPtn && output.match(new RegExp(connectPtn, 'i'))) {
					this.updateFailed(DeliveryError.server, null);
				} else {
					delivery.tries = 0;
					delivery.count = 0;
					if (errorPtn) {
						var regEx = new RegExp(errorPtn, 'i');
						var result = output.match(regEx);
						if (result) errorText = parseStatus(result[1], options);
					}
					this.updateFailed(DeliveryError.notFound, errorText);
				}
			} else {
				delivery.count = 1;
			}
		}

	} else if (isChild) {

		logRequest(1,request);
		if (delivery.usedItemCache) {
			logMessage(1,'Cached URL failed to load');
			delivery.clearItemCache = true;
		} if (!request.responseText) {
			this.updateFailed(DeliveryError.notFound, null);
		}

	} else {

		logRequest(1,request);
		var error = DeliveryError.notFound;
		if (request.status >= 500) {
			error = DeliveryError.server;
		}
		this.updateFailed(error, null);

	}

	this.reqComplete++;
	this.updateItems();

}

function getTrackerEvent (input, options, regEx, statusNo, statusNoExtra, 
	dateNo, timeNo, locNo, locNoExtra, locPtn) {

	if (!input) return null;
	var result = input.match(regEx);
	if (!result) return null;

	var status = result[statusNo];
	status = clearEmptyString(parseStatus(status, options));

	if (statusNoExtra >= 0) {
		var extra = trim(result[statusNoExtra]);
		if (extra) status += ': '+result[statusNoExtra];
	}

	var dateRaw = null;
	if (dateNo) {
		dateRaw = result[dateNo];
		if (timeNo && result[timeNo]) dateRaw += ' '+result[timeNo];
	}

	var location = null;
	if (locNo) {
		location = result[locNo];
		if (locNoExtra >= 0 && result[locNoExtra]) {
			location = (location) ? location+' '+result[locNoExtra] : result[locNoExtra];
		}
	} else if (locPtn) {
		var locRegEx = new RegExp(locPtn, 'i');
		var locResult = status.match(locRegEx);
		if (locResult) {
			location = locResult[1];
			if (locPtn.slice(-1) === '$') {
				status = status.replace(locRegEx, '');
			}
		}
	}
	location = clearEmptyString(location);

	var event = new DeliveryEvent();
	event.status = status;
	event.dateRaw = dateRaw;
	event.location = location;
	return event;

}

function getLanguageFrom (defaultFrom, output) {
	var from = defaultFrom;
	var result = output.match(/<header class='[^']*nav-lang-([a-z]*)[^']*'>/i);
	if (!result) result = output.match(/<html lang="([a-z]{2})-[a-z]+"[^>]*>/i);
	if (result) {
		switch (result[1]) {
			case 'en': from = 'www.amazon.com'; break;
			case 'ja': from = 'www.amazon.co.jp'; break;
			case 'de': from = 'www.amazon.de'; break;
			case 'es': from = 'www.amazon.es'; break;
			case 'fr': from = 'www.amazon.fr'; break;
			case 'it': from = 'www.amazon.it'; break;
		}
		if (defaultFrom != from) logMessage(1, 'Language detected: '+result[1]);
	}
	return from;
}

function parseStatus (string, options) {
	if (typeof string == 'string') {
		switch (options.from) {
			case 'www.amazon.com':
			case 'www.amazon.com/url':
			case 'www.amazon.ca':
			case 'www.amazon.co.uk':
			case 'www.amazon.co.jp':
			case 'www.amazon.de':
			case 'www.amazon.at':
			case 'www.amazon.es':
			case 'www.amazon.fr':
			case 'www.amazon.in':
			case 'www.amazon.it':
				string = string.replace(/\(\)\s*$/, '');
				break;
			case 'www.usps.com':
				string = removeHtmlEntities(string);
				break;
		}
		string = stripHTML(string);
	}
	return string;
}

function parseDate (input, options, future) {

	if (!input) return null;
	var string = trim(input);
	var from = options.from;
	var date = null;
	var isoDate = null;
	var match;
	var timeRegEx = /(, |,? by |,? bis |,? avant |,? entro le |,? antes de las )?([0-9]{2}:[0-9]{2}:[0-9]{2}|[0-9]{1,2}:[0-9]{2} ?(a\.m\.\/p\.m\.|A\.?M\.?|P\.?M\.?|Uhr|Hours|h)?|[0-9]{1,2} ?(am|pm)|[0-9]{1,2}h(?:[0-9]{2})?|[0-9]{1,2}時[0-9]{1,2}分)/i;
	var commonRegEx = /(?:^|[^0-9])([0-9]{1,2})([\/.-])([0-9]{1,2})(\2)([0-9]{4}|[0-9]{2})(?:[^0-9]|$)/i;

	switch (from) {

		case 'www.amazon.com':
		case 'www.amazon.com/url':
		case 'www.amazon.ca':
		case 'www.amazon.co.uk':
		case 'www.amazon.in':
			string = string.replace(timeRegEx,'');
			string = string.replace(/, On time ?$/i, '');
			if (match = string.match(/^([a-z]*,)? ?([a-z]+) ([0-9]+)$/i)) { // com, ca
				date = dateFromParts(null, match[2], match[3], 'en', future);
			} else if (match = string.match(/^([a-z]*,)? ?([0-9]+) ([a-z]+)$/i)) { // uk, in
				date = dateFromParts(null, match[3], match[2], 'en', future);
			} else if (match = string.match(/^([a-z]*,)? ?([a-z]+) ([0-9]+), ([0-9]+)/i)) { // com (new)
				date = dateFromParts(match[4], match[2], match[3], 'en', future);
			} else if (possibleWeekDay(string)) {
				date = dateFromWeekDay(string, 'en');
			}
			break;

		case 'www.amazon.co.jp':
		case 'www.sagawa-exp.co.jp':
			string = string.replace(timeRegEx,'');
			if (match = string.match(/^([0-9]+)月([0-9]+)日(.*)$/i)) {
				date = dateFromParts(null, match[1], match[2], null, future);
			} else if (match = string.match(/^([0-9]+)\/([0-9]+)(\s.*|$)/i)) {
				date = dateFromParts(null, match[1], match[2], null, future);
			} else if (match = string.match(/([0-9]+)年([0-9]+)月([0-9]+)日(.*)/i)) {
				date = dateFromParts(match[1], match[2], match[3]);
			}
			break;

		case 'www.amazon.de':
		case 'www.amazon.at':
			string = string.replace(timeRegEx,'');
			if (match = string.match(/^([a-z]*\.?,)? ?([0-9]+)\.? ([a-z\u00E0-\u00FC]+\.?) ?([0-9]+)?/i)) {
				date = dateFromParts(match[4], match[3], match[2], 'de', future);
			} else if (possibleWeekDay(string)) {
				date = dateFromWeekDay(string, 'de');
			}
			break;

		case 'www.amazon.es':
			string = string.replace(timeRegEx,'');
			if (match = string.match(commonRegEx)) {
				date = dateFromParts(match[5], match[3], match[1]);
			} else if (match = string.match(/^(?:[a-z\u00E0-\u00FC]*, )?([0-9]+) (?:de )?([a-z]+\.?)/i)) {
				date = dateFromParts(null, match[2], match[1], 'es', future);
			} else if (possibleWeekDay(string)) {
				date = dateFromWeekDay(string, 'es');
			}
			break;

		case 'www.amazon.fr':
			string = string.replace(timeRegEx,'');
			if (match = string.match(/^([a-z]*\.?)? ?([0-9]+) ([a-z\u00E0-\u00FC]+) ?$/i)) {
				date = dateFromParts(null, match[3], match[2], 'fr', future);
			} else if (possibleWeekDay(string)) {
				date = dateFromWeekDay(string, 'fr');
			}
			break;

		case 'www.amazon.it':
			string = string.replace(timeRegEx,'');
			if (match = string.match(/^([a-z\u00E0-\u00FC']*\.?)? ?([0-9]+) ([a-z]+) ?$/i)) {
				date = dateFromParts(null, match[3], match[2], 'it', future);
			} else if (possibleWeekDay(string)) {
				date = dateFromWeekDay(string, 'it');
			}
			break;

		case 'www.apple.com':
			if (match = string.match(/^(Sun|Mon|Tue|Wed|Thu|Fri|Sat)?,? ?([a-z]*) ([0-9]{1,2})(?:, ([0-9]{4}))?/i)) {
				date = dateFromParts(match[4], match[2], match[3], 'en', future);
			} else if (match = string.match(/^([0-9]{1,2}) ([a-z]*)/i)) {
				date = dateFromParts(null, match[2], match[1], 'en', future);
			} else if (match = string.match(commonRegEx)) {
				date = dateFromParts(match[5], match[3], match[1]);
			} else if (possibleWeekDay(string)) {
				date = dateFromWeekDay(string, 'en');
			}
			break;

		case 'www.apple.com/japan':
			if (match = string.match(/^([0-9]{4})\/([0-9]{2})\/([0-9]{2})$/)) {
				date = dateFromParts(match[1], match[2], match[3]);
			}
			break;

		case 'www.apple.com/hk':
			if (match = string.match(/^([0-9]+)年([0-9]+)月([0-9]+)日/i)) {
				date = dateFromParts(match[1], match[2], match[3]);
			}
			break;

		case 'www.chronopost.fr':
			var lang = options.lang;
			if (match = string.match(commonRegEx)) {
				if (lang == 'fr') {
					date = dateFromParts(match[5], match[3], match[1]);
				} else {
					date = dateFromParts(match[5], match[1], match[3]);
				}
			}
			break;

		case 'www.dhl.com':
			string = string.replace(/ by End of Day$/,'');
			if (match = string.match(/^[a-z]*, ([a-z]+) ([0-9]+), ([0-9]+)$/i)) {
				date = dateFromParts(match[3], match[1], match[2], 'en');
			} else if (possibleWeekDay(string)) {
				date = dateFromWeekDay(string, 'en');
			}
			break;

		case 'nolp.dhl.de':
		case 'www.poste.it':
			if (isoDate = dateFromISOString(string)) {
				date = isoDate;
			} else if (match = string.match(commonRegEx)) {
				date = dateFromParts(match[5], match[1], match[3]);
			}
			break;

		case 'www.gls-germany.com':
			if (isoDate = dateFromISOString(string)) {
				date = isoDate;
			} else if (match = string.match(commonRegEx)) {
				date = dateFromParts(match[5], match[1], match[3]);
			} else if (match = string.match(/^([0-9]+)-([a-z]+)-([0-9]+)/i)) {
				date = dateFromParts(match[3], match[2], match[1], 'en');
			}
			break;

		case 'www.laposte.fr':
		case 'www.parcelforce.com':
		case 'www.posten.no':
			if (isoDate = dateFromISOString(string)) {
				date = isoDate;
			} else if (match = string.match(commonRegEx)) {
				date = dateFromParts(match[5], match[3], match[1]);
			}
			break;

		case 'www.japanpost.jp/ems':
		case 'www.japanpost.jp/m10':
		case 'www.japanpost.jp/reg':
		case 'www.japanpost.jp/yupack':
			if (match = string.match(/^([0-9]{4})\/([0-9]{2})\/([0-9]{2})/i)) { // ja
				date = dateFromParts(match[1], match[2], match[3]);
			} else if (match = string.match(commonRegEx)) { // en
				date = dateFromParts(match[5], match[1], match[3]);
			}
			break;

		case 'www.ontrac.com':
			if (match = string.match(/^([a-z]+) ([0-9]{1,2}) ([0-9]{4})/i)) {
				date = dateFromParts(match[3], match[1], match[2], 'en');
			} else if (match = string.match(commonRegEx)) { // en
				date = dateFromParts(match[5], match[1], match[3]);
			}
			break;

		case 'www.post.ch':
			string = string.replace(/^&ndash;$/,'');
			if (match = string.match(commonRegEx)) {
				date = dateFromParts(match[5], match[3], match[1]);
			}
			break;

		case 'www.ups.com':
			if (options.locale == 'en_US') {
				if (match = string.match(/^([0-9]{2})\/([0-9]{2})\/([0-9]{4})/)) {
					date = dateFromParts(match[3], match[1], match[2]);
				}
			} else if (match = string.match(commonRegEx)) { // Most locales
				date = dateFromParts(match[5], match[3], match[1]);
			} else if (match = string.match(/^([0-9]{4})\/([0-9]{2})\/([0-9]{2})/i)) { // ja_JP
				date = dateFromParts(match[1], match[2], match[3]);
			}
			break;

		case 'www.ups-mi.net':
			if (match = string.match(/^([a-z]+) ([0-9]{1,2}) ([0-9]{4})$/i)) {
				date = dateFromParts(match[3], match[1], match[2], 'en');
			} else if (match = string.match(/^([0-9]{1,2}) ([a-z]+) ([0-9]{4})/i)) {
				date = dateFromParts(match[3], match[2], match[1], 'en');
			}
			break;

		case 'www.usps.com':
			if (match = string.match(/^([a-z]+) ([0-9]{1,2}), ([0-9]{4})\b/i)) {
				date = dateFromParts(match[3], match[1], match[2], 'en');
			} else if (match = string.match(/^([0-9]{1,2}) ([a-z]+) ([0-9]{4})$/i)) {
				date = dateFromParts(match[3], match[2], match[1], 'en'); // Dashboard
			}
			break;

		case 'www.kuronekoyamato.co.jp':
			if (string == '-') string = '';
			if (match = string.match(/^([0-9]{2})\/([0-9]{2})(?:\s|$)/)) {
				date = dateFromParts(null, match[1], match[2], null, future);
			}
			break;

		case 'www.yodel.co.uk':
			if (match = string.match(/^(?:[a-z]* )?([0-9]+)[a-z]* ([a-z]+)/i)) {
				date = dateFromParts(null, match[2], match[1], 'en', future);
			} else if (possibleWeekDay(string)) {
				date = dateFromWeekDay(string, 'en');
			} else if (match = string.match(commonRegEx)) {
				date = dateFromParts(match[5], match[3], match[1]);
			}
			break;

		case 'www.hlg.de':
		case 'www.tntexpress.com.au':
		case 'www.post.at':
			if (match = string.match(commonRegEx)) {
				date = dateFromParts(match[5], match[3], match[1]);
			}
			break;

		case 'www.fedex.com':
			string = string.replace(/T00:00:00-06:00$/i,'');
			date = dateFromISOString(string);
			break;

		case 'www.aramex.com':
		case 'em.canadapost.ca':
		case 'www.dpd.com':
		case 'www.lasership.com':
		case 'www.postdanmark.dk':
		case 'www.posten.se':
		case 'www.postnl.nl':
		case 'www.purolator.com':
		case 'www.tnt.com':
		case 'www.tnt.com/ref':
			date = dateFromISOString(string);
			break;

	}

	if (date === null) {
		logMessage(1, 'Unrecognized date: '+input+' ('+from+')');
	}

	return date;

}

function endDateInRange (string, flexible) {
	var result = datesInRange(string,flexible);
	return result[1];
}

function datesInRange (string, flexible) {
	string = trim(string);
	var match;
	// Fri, Feb 20 - Wed, Feb 25 by 8pm
	if (flexible && (match = string.match(/(.*) - (.*)/))) return [match[1],match[2]];
	// March 12 - March 22, 17 Dec - 21 Dec
	if (match = string.match(/(.{2,}) - ([^(]+)/i)) return [match[1],match[2]];
	// March 16-22
	if (match = string.match(/([a-z]+) ([0-9]{1,2})-([0-9]{1,2})/i)) return [match[1]+' '+match[2],match[1]+' '+match[3]];
	// 3/16-3/22
	if (match = string.match(/([0-9]{1,2}\/[0-9]{1,2})-([0-9]{1,2}\/[0-9]{1,2})/i)) return [match[1],match[2]];
	// entre le 20 janv et le 25 janv.
	if (match = string.match(/(?:entre )?(le [0-9]+ [a-z]+) et (le [0-9]+ [a-z]+)\.?/i)) return [match[1],match[2]];
	return ['',string];
}

function parseLocation (string, options) {
	if (string) {
		switch (options.from) {
			case 'www.amazon.com':
			case 'www.amazon.com/url':
				string = defaultParseLocation(string);
				string = string.replace(/^US$/i,'United States');
				string = string.replace('Unknown','');
				string = string.replace(/^March Air Reserve Ba,/,'March Air Reserve Base,');
				// China Post
				string = string.replace(/^United States Of America Us([0-9]+)$/i,'$1, United States');
				string = string.replace(/^United States Of America Usus([A-Z]{3})[a-z]$/i,'$1, United States');
				string = string.replace('GuangDongShengShenZhenShi BTYY','ShenZhen, GuangDongSheng');
				// USPS
				string = string.replace(/ ((International |Network )?Distribution Center( Annex)?), /i,', ');
				break;
			case 'www.aramex.com':
				string = defaultParseLocation(string);
				string = string.replace(/ (Branch|Office|Express Hub),/i,',');
				break;
			case 'em.canadapost.ca':
				string = defaultParseLocation(string);
				string = string.replace(/^US([A-Z]{3})[A-Z],? ?(USA)?$/i,'$1, United States');
				string = string.replace(/[0-9]+, ?(Spain|China|South Korea)/i,'$1');
				string = string.replace('VILLE ST-LAURENT, QC','Saint-Laurent, Quebec');
				string = string.replace(/^KRBUSA\b/i,'Busan');
				string = string.replace(/^KRSELB\b/i,'Seoul');
				string = string.replace(/^KRSELE\b/i,'Seoul');
				string = string.replace(/^KRSELT\b/i,'Seoul');
				break;
			case 'www.chronopost.fr':
				string = defaultParseLocation(string);
				string = removeHtmlEntities(string);
				string = string.replace(/^\.+$/,'');
				string = string.replace(/^(Chronopost France|Web Services|Service d'avisage)$/,'');
				string = string.replace(/^([A-Z]+) CHRONOPOST$/i,'$1, FR');
				string = string.replace(/^REUNION CEI$/,'La Réunion, FR');
				string = string.replace(/^NESPRESSO FRANCE SAS$/,''); // Initial location may be a business
				string = string.replace(/(-|,)? (CHRONOPOST|FLOREVEUR)/i,'');
				break;				
			case 'www.dhl.com':
				string = defaultParseLocation(string);
				string = string.replace(/ (HUB|GATEWAY)([,-\s]*)/i,'$2');
				string = string.replace(/^(.*?) - \1$/,'$1');
				string = string.replace(/, THE$/i,'');
				string = string.replace(/ - /i,', ');
				string = string.replace(/CHINA, PEOPLES REPUBLIC$/i,'CHINA');
				string = string.replace('ONTARIO SERVICE AREA, ON, CANADA','ONTARIO, CANADA');
				break;
			case 'nolp.dhl.de':
				string = defaultParseLocation(string);
				string = string.replace(/^HUB /i,'');
				string = string.replace(/^Rüdersdorf(, (Germany|Deutschland))?$/i,'Rüdersdorf bei Berlin$1');
				break;
			case 'www.dpd.com':
				string = defaultParseLocation(string);
				string = string.replace(/^(DPD Datenzentrum|DPD data centre)$/i,'');
				break;
			case 'www.fedex.com':
				string = defaultParseLocation(string);
				string = string.replace(/^FEDEX SMARTPOST /i,'');
				string = string.replace(/(^|\s)(.*?), \2$/i,'$1$2');
				string = string.replace(/^([A-Z\s]+), CA$/,'$1, CA, US');
				break;
			case 'www.kuronekoyamato.co.jp':
				string = defaultParseLocation(string);
				string = string.replace(/支店(（[^）]+）)?$/i,'$1');
				string = string.replace(/(センター|管支店\sサービスセンター|物流システム|ＡＦＣ|ベース店?)$/i,'');
				break;
			case 'www.lasership.com':
				string = defaultParseLocation(string);
				string = string.replace(/^US$/i,'United States');
				string = string.replace('Unknown','');
				break;
			case 'www.ontrac.com':
				string = defaultParseLocation(string);
				string = string.replace(/^([A-Z]+), CA$/i,'$1, CA, US');
				break;
			case 'www.parcelforce.com':
				string = defaultParseLocation(string);
				string = string.replace(/^Delivery Agent - /i,'');
				string = string.replace(/ DEFAULT$/i,'');
				string = string.replace(/^(National|International)$/i,''); // Hub already removed
				break;
			case 'www.post.at':
				string = defaultParseLocation(string);
				string = string.replace(/^ATALLA\b/i,'Allhaming');
				string = string.replace(/^ATVIEA\b/i,'Wien Flughafen');
				string = string.replace(/^ATBIEA\b/i,'Biedermannsdorf');
				string = string.replace(/^ATLNZD\b/i,'Linz');
				string = string.replace(/^ATLUTA\b/i,'Lauterach');
				string = string.replace(/^ATVIEG\b/i,'Wien');
				string = string.replace(/^ATVIEB\b/i,'Wien');
				string = string.replace(/^ATWNFA\b/i,'Wiener Neudorf');
				string = string.replace(/^ATWNFZ\b/i,'Wiener Neudorf');
				string = string.replace(/^ATVIEC\b/i,'Wien');
				string = string.replace(/^ATSZGA\b/i,'Salzburg');
				string = string.replace(/^ATHALA\b/i,'Hall in Tirol');
				string = string.replace(/^ATWLRA\b/i,'Wolfurt');
				var matches = null;
				if (matches = string.match(/^(Postcode|PLZ) (.*)$/i)) {
					string = matches[2];
					if (string.match(/^([1-9][0-9]{3})$/)) {
						if (parseInt(string) <= 1901) string = 'Vienna';
						string += ', Austria';
					}
				}
				break;
			case 'www.post.ch':
				string = defaultParseLocation(string);
				string = string.replace(/^(Swiss Post|[0-9]+) /i,'');
				string = string.replace(/ (Zustellung|Filiale di distribuzione)$/i,'');
				string = string.replace('FRCYMA','Chilly-Mazarin, France');
				string = string.replace('77400','77400 France');
				string = string.replace('SE','Sweden');
				string = string.replace('Asendia USA','USA');
				break;
			case 'www.poste.it':
				string = defaultParseLocation(string);
				string = string.replace(/^([A-Z]{2})$/,'$1, Italia');	
				string = string.replace(/Centro Scambi Internazionale/i,'Linate, Milano, Italia');		
				break;
			case 'www.posten.no':
				string = defaultParseLocation(string);
				string = string.replace(/^[0-9]{4} ([A-ZÆØÅ\s]*)$/,'$1, Norway');
				string = string.replace(/^([0-9]{5})([0-9]{4}), United States$/,'$1-$2, United States');
				
				break;
			case 'www.postdanmark.dk': // www.postnord.dk
			case 'www.posten.se': // www.postnord.se
				string = defaultParseLocation(string);
				// Sweden
				string = string.replace(/^(PostNord,?|Posten ?Företag|Posten)\b ?/,'');
				// Denmark
				string = string.replace(/^(.*?) (Distributionscenter|Pakkecenter - [A-Z]+),/,'$1,');
				string = string.replace(/^Vestegnens Pakkedist, Denmark$/,'Priorparken 385, 2605 Brøndby, Denmark');
				string = string.replace(/^Internationalt Postcenter, Denmark$/,'');
				break;
			case 'www.postnl.nl':
			case 'www.tntpost.nl':
				string = defaultParseLocation(string);
				string = string.replace(/^([0-9]{4} ?[A-Z]{2})( .*)$/,'$1');
				break;
			case 'www.purolator.com':
				string = defaultParseLocation(string);
				string = string.replace(/ Sort Ctr\/ctr Tri/i,'');
				string = string.replace(/^(Purolator|Purolator Freight)$/,'');
				break;
			case 'www.sagawa-exp.co.jp':
				string = defaultParseLocation(string);
				string = string.replace(/中継センター$/,'');
				break;
			case 'www.tnt.com':
			case 'www.tnt.com/ref':
				string = string.replace('Melbourne', 'Melbourne, Australia');
				string = string.replace('MILAN INTL ROAD HUB', 'MILAN, IT');
				string = string.replace('GLOBAL TRANSIT HUB','TNT Global Transit Hub, Netherlands');
				string = defaultParseLocation(string);
				string = string.replace(/^HUB RHEIN MAIN DFT$/i,'Rhein-Main');
				string = string.replace(/^AL BATHA BORDER CUSTOMS$/i,''); // Unknown to MapKit
				string = string.replace('DHAHRAN CAUSEWAY','Dhahran');
				string = string.replace(/\/HEAD OFFICE$/i,'');
				// INTL ROAD HUB, Road Hub, Air Hub: Hub was already removed
				string = string.replace(/ (INTL ROAD|Road|Air|Border Customs|Control Tower)$/i,'');
				break;
			case 'www.tntexpress.com.au':
				string = defaultParseLocation(string);
				string = string.replace(/ (Priority|Riteway)$/i,'');
				break;
			case 'www.ups.com':
				string = defaultParseLocation(string);
				string = string.replace(/\b([A-Z]{2}) US\b/,'$1, United States');
				string = string.replace(/\bHodgkins, IN\b/i,'Hammond, IN');
				string = string.replace(/Herne-Boernig, Germany/i,'Herne-Börnig, Germany');
				string = string.replace(/\bKorea, Republic of\b/i,'South Korea');
				break;
			case 'www.usps.com':
				var city = stringFromMatch(string,'<EventCity>([^<]*)</EventCity>');
				city = city.replace(/^ISC /i,'');
				city = city.replace(/ ?\(USPS\)$/i,'');
				city = city.replace(/ (INT|INT CONTAINER [0-9]+)$/i,'');
				city = city.replace(/ ((INTERNATIONAL |NETWORK )?(POST OFFICE|DISTRIBUTION CENTER( ANNEX)?)) ?$/i,'');
				city = city.replace('LANGLEY HWDC','Heathrow World Distribution Centre');
				var state = stringFromMatch(string,'<EventState>([^<]*)</EventState>');
				var zip = stringFromMatch(string,'<EventZIPCode>([^<]*)</EventZIPCode>');
				var country = stringFromMatch(string,'<EventCountry>([^<]*)</EventCountry>');
				country = country.replace(/\bKOREA REPUBLIC OF\b/i,'South Korea');
				if (city == 'WASHINGTON' && !state) city = 'WASHINGTON DC';
				string = locationFromParts(city,state,zip,country) || string;
				string = removeHtmlEntities(string);
				string = string.replace(/^ON ITS WAY TO .+$/i,'');
				string = string.replace(/^NTX P&DC$/i,'Coppell, TX 75099'); // Or remove P&DC
				string = string.replace(/^NORTH METRO, GA 30026$/i,'Duluth, GA 30026');
				string = string.replace(/ EMS CHINA$/i,' CHINA');
				string = string.replace(/^TOKYO INT( CONTAINER 1)? JAPAN$/i,'Tokyo, Japan');
				string = string.replace('SOUTHERN MD FACILITY, MD 20797','US Post Office, Laurel, MD'); // PO zip code
				string = string.replace('NORTH BAY CA','Petaluma, CA 94999'); // NORTH BAY CA DISTRIBUTION CENTER
				break;
			case 'www.yodel.co.uk':
				string = defaultParseLocation(string);
				string = string.replace(/(Customer| service centre| hub| depot| van| home delivery)$/i,'');
				break;
			default:
				string = defaultParseLocation(string);
				break;
		}
	}
	return string;
}

function defaultParseLocation (string) {
	if (string) {
		string = stripHTML(string);
		string = string.replace(/ (Hub|Depot|Distribution|Distributionsbasis|Paketzentrum|Centre Colis|Base de distribution)$/i,'');
	}
	return string;
}

function locationFromParts (city, state, zip, country) {
	var location = '';
	if (city) location += city;
	if (state) {
		if (location) location += ', ';
		location += state;
	}
	if (zip) {
		if (location) location += ' ';
		location += zip;
	}
	if (country) {
		if (location) location += ', ';
		location += country;
	}
	return location;
}

function locationFromStatus (status, from) {
	if (!status) return '';
	var pattern = /Die Sendung ist im Hermes Verteilzentrum (.*?) eingetroffen/i;
	var location = status.match(pattern);
	if (!location) return '';
	return location[1];
}
